@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -52,26 +52,31 @@
 
 QT_BEGIN_NAMESPACE
 
-char         appFileName[MAX_PATH];                // application file name
-char         theAppName[MAX_PATH];                        // application name
-HINSTANCE appInst        = 0;                // handle to app instance
-HINSTANCE appPrevInst        = 0;                // handle to prev app instance
-int appCmdShow = 0;
 bool usingWinMain = false;  // whether the qWinMain() is used or not
+int appCmdShow = 0;
 
 Q_CORE_EXPORT HINSTANCE qWinAppInst()                // get Windows app handle
 {
-    return appInst;
+    return GetModuleHandle(0);
 }
 
 Q_CORE_EXPORT HINSTANCE qWinAppPrevInst()                // get Windows prev app handle
 {
-    return appPrevInst;
+    return 0;
 }
 
 Q_CORE_EXPORT int qWinAppCmdShow()                        // get main window show command
 {
+#if defined(Q_OS_WINCE)
     return appCmdShow;
+#else
+    STARTUPINFO startupInfo;
+    GetStartupInfo(&startupInfo);
+
+    return (startupInfo.dwFlags & STARTF_USESHOWWINDOW)
+        ? startupInfo.wShowWindow
+        : SW_SHOWDEFAULT;
+#endif
 }
 
 Q_CORE_EXPORT QString qAppFileName()                // get application file name
@@ -115,25 +120,6 @@ Q_CORE_EXPORT QString qAppFileName()                // get application file name
     return res;
 }
 
-void set_winapp_name()
-{
-    static bool already_set = false;
-    if (!already_set) {
-        already_set = true;
-
-        QString moduleName = qAppFileName();
-
-        QByteArray filePath = moduleName.toLocal8Bit();
-        QByteArray fileName = QFileInfo(moduleName).baseName().toLocal8Bit();
-
-        memcpy(appFileName, filePath.constData(), filePath.length());
-        memcpy(theAppName, fileName.constData(), fileName.length());
-
-        if (appInst == 0)
-            appInst = GetModuleHandle(0);
-    }
-}
-
 QString QCoreApplicationPrivate::appName() const
 {
     return QFileInfo(qAppFileName()).baseName();
@@ -198,20 +184,17 @@ void qWinMain(HINSTANCE instance, HINSTANCE prevInstance, LPSTR cmdParam,
     already_called = true;
     usingWinMain = true;
 
-  // Install default debug handler
-
+    // Install default debug handler
     qInstallMsgHandler(qWinMsgHandler);
 
-  // Create command line
-
+    // Create command line
     argv = qWinCmdLine<char>(cmdParam, int(strlen(cmdParam)), argc);
-    // Get Windows parameters
 
-    appInst = instance;
-    appPrevInst = prevInstance;
     appCmdShow = cmdShow;
 
-    set_winapp_name();
+    // Ignore Windows parameters
+    Q_UNUSED(instance);
+    Q_UNUSED(prevInstance);
 }
 
 /*!
@@ -1021,14 +1004,14 @@ QString decodeMSG(const MSG& msg)
                 LPWINDOWPOS winPos = (LPWINDOWPOS)lParam;
                 if (!winPos)
                     break;
-                QString hwndAfter = valueCheck((uint)winPos->hwndInsertAfter,
-                                          FLAG_STRING((uint)HWND_BOTTOM,    "HWND_BOTTOM"),
-                                          FLAG_STRING((int)HWND_NOTOPMOST, "HWND_NOTOPMOST"),
-                                          FLAG_STRING((uint)HWND_TOP,       "HWND_TOP"),
-                                          FLAG_STRING((int)HWND_TOPMOST,   "HWND_TOPMOST"),
+                QString hwndAfter = valueCheck(quint64(winPos->hwndInsertAfter),
+                                          FLAG_STRING((qptrdiff)HWND_BOTTOM,    "HWND_BOTTOM"),
+                                          FLAG_STRING((qptrdiff)HWND_NOTOPMOST, "HWND_NOTOPMOST"),
+                                          FLAG_STRING((qptrdiff)HWND_TOP,       "HWND_TOP"),
+                                          FLAG_STRING((qptrdiff)HWND_TOPMOST,   "HWND_TOPMOST"),
                                           FLAG_STRING());
-                if (hwndAfter.size() == 0)
-                    hwndAfter = QString::number((uint)winPos->hwndInsertAfter, 16);
+                if (hwndAfter.isEmpty())
+                    hwndAfter = QString::number((quintptr)winPos->hwndInsertAfter, 16);
                 QString flags = flagCheck(winPos->flags,
                                           FLGSTR(SWP_DRAWFRAME),
                                           FLGSTR(SWP_FRAMECHANGED),
