@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -185,12 +185,12 @@ void QDirModelPrivate::invalidate()
 
 /*!
     \class QDirModel
-
+    \obsolete
     \brief The QDirModel class provides a data model for the local filesystem.
 
     \ingroup model-view
 
-    \note The usage of QDirModel is not recommended anymore. The
+    The usage of QDirModel is not recommended anymore. The
     QFileSystemModel class is a more performant alternative.
 
     This class provides access to the local filesystem, providing functions
@@ -201,25 +201,6 @@ void QDirModelPrivate::invalidate()
     QDirModel keeps a cache with file information. The cache needs to be
     updated with refresh().
 
-    A directory model that displays the contents of a default directory
-    is usually constructed with a parent object:
-
-    \snippet doc/src/snippets/shareddirmodel/main.cpp 2
-
-    A tree view can be used to display the contents of the model
-
-    \snippet doc/src/snippets/shareddirmodel/main.cpp 4
-
-    and the contents of a particular directory can be displayed by
-    setting the tree view's root index:
-
-    \snippet doc/src/snippets/shareddirmodel/main.cpp 7
-
-    The view's root index can be used to control how much of a
-    hierarchical model is displayed. QDirModel provides a convenience
-    function that returns a suitable model index for a path to a
-    directory within the model.
-
     QDirModel can be accessed using the standard interface provided by
     QAbstractItemModel, but it also provides some convenience functions
     that are specific to a directory model. The fileInfo() and isDir()
@@ -1182,12 +1163,18 @@ QFileInfo QDirModel::fileInfo(const QModelIndex &index) const
 
 void QDirModelPrivate::init()
 {
+    Q_Q(QDirModel);
     filters = QDir::AllEntries | QDir::NoDotAndDotDot;
     sort = QDir::Name;
     nameFilters << QLatin1String("*");
     root.parent = 0;
     root.info = QFileInfo();
     clear(&root);
+    QHash<int, QByteArray> roles = q->roleNames();
+    roles.insertMulti(QDirModel::FileIconRole, "fileIcon"); // == Qt::decoration
+    roles.insert(QDirModel::FilePathRole, "filePath");
+    roles.insert(QDirModel::FileNameRole, "fileName");
+    q->setRoleNames(roles);
 }
 
 QDirModelPrivate::QDirNode *QDirModelPrivate::node(int row, QDirNode *parent) const
