@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -212,6 +212,7 @@ bool QItemSelectionRange::intersects(const QItemSelectionRange &other) const
 {
     return (isValid() && other.isValid()
             && parent() == other.parent()
+            && model() == other.model()
             && ((top() <= other.top() && bottom() >= other.top())
                 || (top() >= other.top() && top() <= other.bottom()))
             && ((left() <= other.left() && right() >= other.left())
@@ -291,6 +292,27 @@ static void indexesFromRange(const QItemSelectionRange &range, QModelIndexList &
     }
 }
 
+/*!
+    Returns true if the selection range contains no selectable item
+    \since 4.7
+*/
+
+bool QItemSelectionRange::isEmpty() const
+{
+    if (!isValid() || !model())
+        return true;
+
+    for (int column = left(); column <= right(); ++column) {
+        for (int row = top(); row <= bottom(); ++row) {
+            QModelIndex index = model()->index(row, column, parent());
+            Qt::ItemFlags flags = model()->flags(index);
+            if ((flags & Qt::ItemIsSelectable) && (flags & Qt::ItemIsEnabled))
+                return false;
+        }
+    }
+    return true;
+}
+
 /*!
     Returns the list of model index items stored in the selection.
 */
@@ -487,7 +509,7 @@ void QItemSelection::merge(const QItemSelection &other, QItemSelectionModel::Sel
 void QItemSelection::split(const QItemSelectionRange &range,
                            const QItemSelectionRange &other, QItemSelection *result)
 {
-    if (range.parent() != other.parent())
+    if (range.parent() != other.parent() || range.model() != other.model())
         return;
 
     QModelIndex parent = other.parent();
@@ -527,6 +549,27 @@ void QItemSelection::split(const QItemSelectionRange &range,
     }
 }
 
+
+void QItemSelectionModelPrivate::initModel(QAbstractItemModel *model)
+{
+    this->model = model;
+    if (model) {
+        Q_Q(QItemSelectionModel);
+        QObject::connect(model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
+                q, SLOT(_q_rowsAboutToBeRemoved(QModelIndex,int,int)));
+        QObject::connect(model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
+                q, SLOT(_q_columnsAboutToBeRemoved(QModelIndex,int,int)));
+        QObject::connect(model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
+                q, SLOT(_q_rowsAboutToBeInserted(QModelIndex,int,int)));
+        QObject::connect(model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
+                q, SLOT(_q_columnsAboutToBeInserted(QModelIndex,int,int)));
+        QObject::connect(model, SIGNAL(layoutAboutToBeChanged()),
+                q, SLOT(_q_layoutAboutToBeChanged()));
+        QObject::connect(model, SIGNAL(layoutChanged()),
+                q, SLOT(_q_layoutChanged()));
+    }
+}
+
 /*!
     \internal
 
@@ -592,6 +635,7 @@ void QItemSelectionModelPrivate::_q_rowsAboutToBeRemoved(const QModelIndex &pare
     }
 
     QItemSelection deselected;
+    QItemSelection newParts;
     QItemSelection::iterator it = ranges.begin();
     while (it != ranges.end()) {
         if (it->topLeft().parent() != parent) {  // Check parents until reaching root or contained in range
@@ -617,13 +661,20 @@ void QItemSelectionModelPrivate::_q_rowsAboutToBeRemoved(const QModelIndex &pare
             deselected.append(QItemSelectionRange(model->index(start, it->right(), it->parent()), it->bottomRight()));
             *it = QItemSelectionRange(it->topLeft(), model->index(start - 1, it->right(), it->parent()));
             ++it;
-        } else {
-            if (it->top() < start && end < it->bottom())  // Middle intersection (do nothing)
-                deselected.append(QItemSelectionRange(model->index(start, it->right(), it->parent()),
-                                                      model->index(end, it->left(), it->parent())));
+        } else if (it->top() < start && end < it->bottom()) { // Middle intersection
+            // If the parent contains (1, 2, 3, 4, 5, 6, 7, 8) and [3, 4, 5, 6] is selected,
+            // and [4, 5] is removed, we need to split [3, 4, 5, 6] into [3], [4, 5] and [6].
+            // [4, 5] is appended to deselected, and [3] and [6] remain part of the selection
+            // in ranges.
+            const QItemSelectionRange removedRange(model->index(start, it->right(), it->parent()),
+                                                    model->index(end, it->left(), it->parent()));
+            deselected.append(removedRange);
+            QItemSelection::split(*it, removedRange, &newParts);
+            it = ranges.erase(it);
+        } else
             ++it;
-       }
     }
+    ranges.append(newParts);
 
     if (!deselected.isEmpty())
         emit q->selectionChanged(QItemSelection(), deselected);
@@ -730,7 +781,7 @@ void QItemSelectionModelPrivate::_q_layoutAboutToBeChanged()
     savedPersistentIndexes.clear();
     savedPersistentCurrentIndexes.clear();
 
-    // optimisation for when all indexes are selected
+    // optimization for when all indexes are selected
     // (only if there is lots of items (1000) because this is not entirely correct)
     if (ranges.isEmpty() && currentSelection.count() == 1) {
         QItemSelectionRange range = currentSelection.first();
@@ -793,6 +844,10 @@ static QItemSelection mergeIndexes(const QList<QPersistentModelIndex> &indexes)
         while (++i < colSpans.count()) {
             QModelIndex nextTl = colSpans.at(i).topLeft();
             QModelIndex nextBr = colSpans.at(i).bottomRight();
+
+            if (nextTl.parent() != tl.parent())
+                break; // we can't merge selection ranges from different parents
+
             if ((nextTl.column() == prevTl.column()) && (nextBr.column() == br.column())
                 && (nextTl.row() == prevTl.row() + 1) && (nextBr.row() == br.row() + 1)) {
                 br = nextBr;
@@ -890,21 +945,7 @@ void QItemSelectionModelPrivate::_q_layoutChanged()
 QItemSelectionModel::QItemSelectionModel(QAbstractItemModel *model)
     : QObject(*new QItemSelectionModelPrivate, model)
 {
-    d_func()->model = model;
-    if (model) {
-        connect(model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeRemoved(QModelIndex,int,int)));
-        connect(model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeRemoved(QModelIndex,int,int)));
-        connect(model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeInserted(QModelIndex,int,int)));
-        connect(model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeInserted(QModelIndex,int,int)));
-        connect(model, SIGNAL(layoutAboutToBeChanged()),
-                this, SLOT(_q_layoutAboutToBeChanged()));
-        connect(model, SIGNAL(layoutChanged()),
-                this, SLOT(_q_layoutChanged()));
-    }
+    d_func()->initModel(model);
 }
 
 /*!
@@ -913,21 +954,7 @@ QItemSelectionModel::QItemSelectionModel(QAbstractItemModel *model)
 QItemSelectionModel::QItemSelectionModel(QAbstractItemModel *model, QObject *parent)
     : QObject(*new QItemSelectionModelPrivate, parent)
 {
-    d_func()->model = model;
-    if (model) {
-        connect(model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeRemoved(QModelIndex,int,int)));
-        connect(model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeRemoved(QModelIndex,int,int)));
-        connect(model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeInserted(QModelIndex,int,int)));
-        connect(model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeInserted(QModelIndex,int,int)));
-        connect(model, SIGNAL(layoutAboutToBeChanged()),
-                this, SLOT(_q_layoutAboutToBeChanged()));
-        connect(model, SIGNAL(layoutChanged()),
-                this, SLOT(_q_layoutChanged()));
-    }
+    d_func()->initModel(model);
 }
 
 /*!
@@ -936,21 +963,7 @@ QItemSelectionModel::QItemSelectionModel(QAbstractItemModel *model, QObject *par
 QItemSelectionModel::QItemSelectionModel(QItemSelectionModelPrivate &dd, QAbstractItemModel *model)
     : QObject(dd, model)
 {
-    d_func()->model = model;
-    if (model) {
-        connect(model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeRemoved(QModelIndex,int,int)));
-        connect(model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeRemoved(QModelIndex,int,int)));
-        connect(model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeInserted(QModelIndex,int,int)));
-        connect(model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeInserted(QModelIndex,int,int)));
-        connect(model, SIGNAL(layoutAboutToBeChanged()),
-                this, SLOT(_q_layoutAboutToBeChanged()));
-        connect(model, SIGNAL(layoutChanged()),
-                this, SLOT(_q_layoutChanged()));
-    }
+    dd.initModel(model);
 }
 
 /*!
@@ -958,21 +971,6 @@ QItemSelectionModel::QItemSelectionModel(QItemSelectionModelPrivate &dd, QAbstra
 */
 QItemSelectionModel::~QItemSelectionModel()
 {
-    Q_D(QItemSelectionModel);
-    if (d->model) {
-        disconnect(d->model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeRemoved(QModelIndex,int,int)));
-        disconnect(d->model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeRemoved(QModelIndex,int,int)));
-        disconnect(d->model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_rowsAboutToBeInserted(QModelIndex,int,int)));
-        disconnect(d->model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
-                this, SLOT(_q_columnsAboutToBeInserted(QModelIndex,int,int)));
-        disconnect(d->model, SIGNAL(layoutAboutToBeChanged()),
-                this, SLOT(_q_layoutAboutToBeChanged()));
-        disconnect(d->model, SIGNAL(layoutChanged()),
-                this, SLOT(_q_layoutChanged()));
-    }
 }
 
 /*!
@@ -1070,6 +1068,19 @@ void QItemSelectionModel::select(const QItemSelection &selection, QItemSelection
 
     // store old selection
     QItemSelection sel = selection;
+    // If d->ranges is non-empty when the source model is reset the persistent indexes
+    // it contains will be invalid. We can't clear them in a modelReset slot because that might already
+    // be too late if another model observer is connected to the same modelReset slot and is invoked first
+    // it might call select() on this selection model before any such QItemSelectionModelPrivate::_q_modelReset() slot
+    // is invoked, so it would not be cleared yet. We clear it invalid ranges in it here.
+    QItemSelection::iterator it = d->ranges.begin();
+    while (it != d->ranges.end()) {
+      if (!it->isValid())
+        it = d->ranges.erase(it);
+      else
+        ++it;
+    }
+
     QItemSelection old = d->ranges;
     old.merge(d->currentSelection, d->currentCommand);
 
@@ -1134,11 +1145,8 @@ void QItemSelectionModel::clearSelection()
     Q_D(QItemSelectionModel);
     if (d->ranges.count() == 0 && d->currentSelection.count() == 0)
         return;
-    QItemSelection selection = d->ranges;
-    selection.merge(d->currentSelection, d->currentCommand);
-    d->ranges.clear();
-    d->currentSelection.clear();
-    emit selectionChanged(QItemSelection(), selection);
+
+    select(QItemSelection(), Clear);
 }
 
 
