@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -156,10 +156,8 @@ void QDesktopWidgetPrivate::init(QDesktopWidget *that)
 
 #ifndef Q_OS_WINCE
     QSystemLibrary user32Lib(QLatin1String("user32"));
-    if (user32Lib.load()) {
-        enumDisplayMonitors = (EnumFunc)user32Lib.resolve("EnumDisplayMonitors");
-        getMonitorInfo = (InfoFunc)user32Lib.resolve("GetMonitorInfoW");
-    }
+    enumDisplayMonitors = (EnumFunc)user32Lib.resolve("EnumDisplayMonitors");
+    getMonitorInfo = (InfoFunc)user32Lib.resolve("GetMonitorInfoW");
 
     if (!enumDisplayMonitors || !getMonitorInfo) {
         screenCount = GetSystemMetrics(80);  // SM_CMONITORS
@@ -174,11 +172,9 @@ void QDesktopWidgetPrivate::init(QDesktopWidget *that)
     getMonitorInfo = 0;
 #else
     QSystemLibrary coreLib(QLatin1String("coredll"));
-    if (coreLib.load()) {
-        // CE >= 4.0 case
-        enumDisplayMonitors = (EnumFunc)coreLib.resolve("EnumDisplayMonitors");
-        getMonitorInfo = (InfoFunc)coreLib.resolve("GetMonitorInfo");
-    }
+    // CE >= 4.0 case
+    enumDisplayMonitors = (EnumFunc)coreLib.resolve("EnumDisplayMonitors");
+    getMonitorInfo = (InfoFunc)coreLib.resolve("GetMonitorInfo");
 
     if ((!enumDisplayMonitors || !getMonitorInfo)) {
         screenCount = GetSystemMetrics(SM_CMONITORS);
