@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -78,12 +78,12 @@ QT_BEGIN_NAMESPACE
     names.
 
     \table
+    \header
+    \o RGB \o HSV \o CMYK
     \row
     \o \inlineimage qcolor-rgb.png
     \o \inlineimage qcolor-hsv.png
     \o \inlineimage qcolor-cmyk.png
-    \header
-    \o RGB \o HSV \o CMYK
     \endtable
 
     The QColor constructor creates the color based on RGB values.  To
@@ -133,7 +133,7 @@ QT_BEGIN_NAMESPACE
     QColor is platform and device independent. The QColormap class
     maps the color to the hardware.
 
-    For more information about painting in general, see \l{The Paint
+    For more information about painting in general, see the \l{Paint
     System} documentation.
 
     \tableofcontents
@@ -531,26 +531,49 @@ QString QColor::name() const
 */
 
 void QColor::setNamedColor(const QString &name)
+{
+    setColorFromString(name);
+}
+
+/*!
+   \since 4.7
+
+   Returns true if the \a name is a valid color name and can
+   be used to construct a valid QColor object, otherwise returns
+   false.
+
+   It uses the same algorithm used in setNamedColor().
+
+   \sa setNamedColor()
+*/
+bool QColor::isValidColor(const QString &name)
+{
+    return !name.isEmpty() && QColor().setColorFromString(name);
+}
+
+bool QColor::setColorFromString(const QString &name)
 {
     if (name.isEmpty()) {
         invalidate();
-        return;
+        return true;
     }
 
     if (name.startsWith(QLatin1Char('#'))) {
         QRgb rgb;
         if (qt_get_hex_rgb(name.constData(), name.length(), &rgb)) {
             setRgb(rgb);
+            return true;
         } else {
             invalidate();
+            return false;
         }
-        return;
     }
 
 #ifndef QT_NO_COLORNAMES
     QRgb rgb;
     if (qt_get_named_rgb(name.constData(), name.length(), &rgb)) {
         setRgba(rgb);
+        return true;
     } else
 #endif
     {
@@ -561,11 +584,12 @@ void QColor::setNamedColor(const QString &name)
             && QX11Info::display()
             && XParseColor(QX11Info::display(), QX11Info::appColormap(), name.toLatin1().constData(), &result)) {
             setRgb(result.red >> 8, result.green >> 8, result.blue >> 8);
+            return true;
         } else
 #endif
         {
-            qWarning("QColor::setNamedColor: Unknown color name '%s'", name.toLatin1().constData());
             invalidate();
+            return false;
         }
     }
 }
@@ -604,7 +628,7 @@ void QColor::getHsvF(qreal *h, qreal *s, qreal *v, qreal *a) const
         return;
     }
 
-    *h = ct.ahsv.hue == USHRT_MAX ? -1.0 : ct.ahsv.hue / 36000.0;
+    *h = ct.ahsv.hue == USHRT_MAX ? qreal(-1.0) : ct.ahsv.hue / qreal(36000.0);
     *s = ct.ahsv.saturation / qreal(USHRT_MAX);
     *v = ct.ahsv.value / qreal(USHRT_MAX);
 
@@ -651,17 +675,17 @@ void QColor::getHsv(int *h, int *s, int *v, int *a) const
 */
 void QColor::setHsvF(qreal h, qreal s, qreal v, qreal a)
 {
-    if (((h < 0.0 || h > 1.0) && h != -1.0)
-        || (s < 0.0 || s > 1.0)
-        || (v < 0.0 || v > 1.0)
-        || (a < 0.0 || a > 1.0)) {
+    if (((h < qreal(0.0) || h > qreal(1.0)) && h != qreal(-1.0))
+        || (s < qreal(0.0) || s > qreal(1.0))
+        || (v < qreal(0.0) || v > qreal(1.0))
+        || (a < qreal(0.0) || a > qreal(1.0))) {
         qWarning("QColor::setHsvF: HSV parameters out of range");
         return;
     }
 
     cspec = Hsv;
     ct.ahsv.alpha      = qRound(a * USHRT_MAX);
-    ct.ahsv.hue        = h == -1.0 ? USHRT_MAX : qRound(h * 36000);
+    ct.ahsv.hue        = h == qreal(-1.0) ? USHRT_MAX : qRound(h * 36000);
     ct.ahsv.saturation = qRound(s * USHRT_MAX);
     ct.ahsv.value      = qRound(v * USHRT_MAX);
     ct.ahsv.pad        = 0;
@@ -715,7 +739,7 @@ void QColor::getHslF(qreal *h, qreal *s, qreal *l, qreal *a) const
         return;
     }
 
-    *h = ct.ahsl.hue == USHRT_MAX ? -1.0 : ct.ahsl.hue / 36000.0;
+    *h = ct.ahsl.hue == USHRT_MAX ? qreal(-1.0) : ct.ahsl.hue / qreal(36000.0);
     *s = ct.ahsl.saturation / qreal(USHRT_MAX);
     *l = ct.ahsl.lightness / qreal(USHRT_MAX);
 
@@ -765,17 +789,17 @@ void QColor::getHsl(int *h, int *s, int *l, int *a) const
 */
 void QColor::setHslF(qreal h, qreal s, qreal l, qreal a)
 {
-    if (((h < 0.0 || h > 1.0) && h != -1.0)
-        || (s < 0.0 || s > 1.0)
-        || (l < 0.0 || l > 1.0)
-        || (a < 0.0 || a > 1.0)) {
+    if (((h < qreal(0.0) || h > qreal(1.0)) && h != qreal(-1.0))
+        || (s < qreal(0.0) || s > qreal(1.0))
+        || (l < qreal(0.0) || l > qreal(1.0))
+        || (a < qreal(0.0) || a > qreal(1.0))) {
         qWarning("QColor::setHsvF: HSV parameters out of range");
         return;
     }
 
     cspec = Hsl;
     ct.ahsl.alpha      = qRound(a * USHRT_MAX);
-    ct.ahsl.hue        = h == -1.0 ? USHRT_MAX : qRound(h * 36000);
+    ct.ahsl.hue        = h == qreal(-1.0) ? USHRT_MAX : qRound(h * 36000);
     ct.ahsl.saturation = qRound(s * USHRT_MAX);
     ct.ahsl.lightness  = qRound(l * USHRT_MAX);
     ct.ahsl.pad        = 0;
@@ -884,10 +908,10 @@ void QColor::getRgb(int *r, int *g, int *b, int *a) const
 */
 void QColor::setRgbF(qreal r, qreal g, qreal b, qreal a)
 {
-    if (r < 0.0 || r > 1.0
-        || g < 0.0 || g > 1.0
-        || b < 0.0 || b > 1.0
-        || a < 0.0 || a > 1.0) {
+    if (r < qreal(0.0) || r > qreal(1.0)
+        || g < qreal(0.0) || g > qreal(1.0)
+        || b < qreal(0.0) || b > qreal(1.0)
+        || a < qreal(0.0) || a > qreal(1.0)) {
         qWarning("QColor::setRgbF: RGB parameters out of range");
         invalidate();
         return;
@@ -934,8 +958,7 @@ void QColor::setRgb(int r, int g, int b, int a)
 /*!
     \fn QRgb QColor::rgba() const
 
-    Returns the RGB value of the color. Unlike rgb(), the alpha is not
-    stripped.
+    Returns the RGB value of the color, including its alpha.
 
     For an invalid color, the alpha value of the returned color is unspecified.
 
@@ -950,8 +973,7 @@ QRgb QColor::rgba() const
 }
 
 /*!
-    Sets the RGBA value to \a rgba. Unlike setRgb(QRgb rgb), this function does
-    not ignore the alpha.
+    Sets the RGB value to \a rgba, including its alpha.
 
     \sa rgba(), rgb()
 */
@@ -968,8 +990,7 @@ void QColor::setRgba(QRgb rgba)
 /*!
     \fn QRgb QColor::rgb() const
 
-    Returns the RGB value of the color. The alpha is stripped for
-    compatibility.
+    Returns the RGB value of the color. The alpha value is opaque.
 
     \sa getRgb(), rgba()
 */
@@ -983,7 +1004,7 @@ QRgb QColor::rgb() const
 /*!
     \overload
 
-    Sets the RGB value to \a rgb, ignoring the alpha.
+    Sets the RGB value to \a rgb. The alpha value is set to opaque.
 */
 void QColor::setRgb(QRgb rgb)
 {
@@ -1300,7 +1321,7 @@ qreal QColor::hsvHueF() const
 {
     if (cspec != Invalid && cspec != Hsv)
         return toHsv().hueF();
-    return ct.ahsv.hue == USHRT_MAX ? -1.0 : ct.ahsv.hue / 36000.0;
+    return ct.ahsv.hue == USHRT_MAX ? qreal(-1.0) : ct.ahsv.hue / qreal(36000.0);
 }
 
 /*!
@@ -1396,7 +1417,7 @@ qreal QColor::hslHueF() const
 {
     if (cspec != Invalid && cspec != Hsl)
         return toHsl().hslHueF();
-    return ct.ahsl.hue == USHRT_MAX ? -1.0 : ct.ahsl.hue / 36000.0;
+    return ct.ahsl.hue == USHRT_MAX ? qreal(-1.0) : ct.ahsl.hue / qreal(36000.0);
 }
 
 /*!
@@ -1557,15 +1578,15 @@ QColor QColor::toRgb() const
             }
 
             // chromatic case
-            const qreal h = ct.ahsv.hue == 36000 ? 0 : ct.ahsv.hue / 6000.;
+            const qreal h = ct.ahsv.hue == 36000 ? 0 : ct.ahsv.hue / qreal(6000.);
             const qreal s = ct.ahsv.saturation / qreal(USHRT_MAX);
             const qreal v = ct.ahsv.value / qreal(USHRT_MAX);
             const int i = int(h);
             const qreal f = h - i;
-            const qreal p = v * (1.0 - s);
+            const qreal p = v * (qreal(1.0) - s);
 
             if (i & 1) {
-                const qreal q = v * (1.0 - (s * f));
+                const qreal q = v * (qreal(1.0) - (s * f));
 
                 switch (i) {
                 case 1:
@@ -1585,7 +1606,7 @@ QColor QColor::toRgb() const
                     break;
                 }
             } else {
-                const qreal t = v * (1.0 - (s * (1.0 - f)));
+                const qreal t = v * (qreal(1.0) - (s * (qreal(1.0) - f)));
 
                 switch (i) {
                 case 0:
@@ -1617,7 +1638,7 @@ QColor QColor::toRgb() const
                 color.ct.argb.red = color.ct.argb.green = color.ct.argb.blue = 0;
             } else {
                 // chromatic case
-                const qreal h = ct.ahsl.hue == 36000 ? 0 : ct.ahsl.hue / 36000.;
+                const qreal h = ct.ahsl.hue == 36000 ? 0 : ct.ahsl.hue / qreal(36000.);
                 const qreal s = ct.ahsl.saturation / qreal(USHRT_MAX);
                 const qreal l = ct.ahsl.lightness / qreal(USHRT_MAX);
 
@@ -1661,9 +1682,9 @@ QColor QColor::toRgb() const
             const qreal y = ct.acmyk.yellow / qreal(USHRT_MAX);
             const qreal k = ct.acmyk.black / qreal(USHRT_MAX);
 
-            color.ct.argb.red   = qRound((1.0 - (c * (1.0 - k) + k)) * USHRT_MAX);
-            color.ct.argb.green = qRound((1.0 - (m * (1.0 - k) + k)) * USHRT_MAX);
-            color.ct.argb.blue  = qRound((1.0 - (y * (1.0 - k) + k)) * USHRT_MAX);
+            color.ct.argb.red   = qRound((qreal(1.0) - (c * (qreal(1.0) - k) + k)) * USHRT_MAX);
+            color.ct.argb.green = qRound((qreal(1.0) - (m * (qreal(1.0) - k) + k)) * USHRT_MAX);
+            color.ct.argb.blue  = qRound((qreal(1.0) - (y * (qreal(1.0) - k) + k)) * USHRT_MAX);
             break;
         }
     default:
@@ -1715,15 +1736,15 @@ QColor QColor::toHsv() const
         if (qFuzzyCompare(r, max)) {
             hue = ((g - b) /delta);
         } else if (qFuzzyCompare(g, max)) {
-            hue = (2.0 + (b - r) / delta);
+            hue = (qreal(2.0) + (b - r) / delta);
         } else if (qFuzzyCompare(b, max)) {
-            hue = (4.0 + (r - g) / delta);
+            hue = (qreal(4.0) + (r - g) / delta);
         } else {
             Q_ASSERT_X(false, "QColor::toHsv", "internal error");
         }
-        hue *= 60.0;
-        if (hue < 0.0)
-            hue += 360.0;
+        hue *= qreal(60.0);
+        if (hue < qreal(0.0))
+            hue += qreal(360.0);
         color.ct.ahsv.hue = qRound(hue * 100);
     }
 
@@ -1759,7 +1780,7 @@ QColor QColor::toHsl() const
     color.ct.ahsl.lightness = qRound(lightness * USHRT_MAX);
     if (qFuzzyIsNull(delta)) {
         // achromatic case, hue is undefined
-        color.ct.ahsl.hue = 0;
+        color.ct.ahsl.hue = USHRT_MAX;
         color.ct.ahsl.saturation = 0;
     } else {
         // chromatic case
@@ -1771,15 +1792,15 @@ QColor QColor::toHsl() const
         if (qFuzzyCompare(r, max)) {
             hue = ((g - b) /delta);
         } else if (qFuzzyCompare(g, max)) {
-            hue = (2.0 + (b - r) / delta);
+            hue = (qreal(2.0) + (b - r) / delta);
         } else if (qFuzzyCompare(b, max)) {
-            hue = (4.0 + (r - g) / delta);
+            hue = (qreal(4.0) + (r - g) / delta);
         } else {
             Q_ASSERT_X(false, "QColor::toHsv", "internal error");
         }
-        hue *= 60.0;
-        if (hue < 0.0)
-            hue += 360.0;
+        hue *= qreal(60.0);
+        if (hue < qreal(0.0))
+            hue += qreal(360.0);
         color.ct.ahsl.hue = qRound(hue * 100);
     }
 
@@ -1807,17 +1828,17 @@ QColor QColor::toCmyk() const
     const qreal r = ct.argb.red   / qreal(USHRT_MAX);
     const qreal g = ct.argb.green / qreal(USHRT_MAX);
     const qreal b = ct.argb.blue  / qreal(USHRT_MAX);
-    qreal c = 1.0 - r;
-    qreal m = 1.0 - g;
-    qreal y = 1.0 - b;
+    qreal c = qreal(1.0) - r;
+    qreal m = qreal(1.0) - g;
+    qreal y = qreal(1.0) - b;
 
     // cmy -> cmyk
     const qreal k = qMin(c, qMin(m, y));
 
     if (!qFuzzyIsNull(k - 1)) {
-        c = (c - k) / (1.0 - k);
-        m = (m - k) / (1.0 - k);
-        y = (y - k) / (1.0 - k);
+        c = (c - k) / (qreal(1.0) - k);
+        m = (m - k) / (qreal(1.0) - k);
+        y = (y - k) / (qreal(1.0) - k);
     }
 
     color.ct.acmyk.cyan    = qRound(c * USHRT_MAX);
@@ -1920,10 +1941,10 @@ QColor QColor::fromRgb(int r, int g, int b, int a)
 */
 QColor QColor::fromRgbF(qreal r, qreal g, qreal b, qreal a)
 {
-    if (r < 0.0 || r > 1.0
-        || g < 0.0 || g > 1.0
-        || b < 0.0 || b > 1.0
-        || a < 0.0 || a > 1.0) {
+    if (r < qreal(0.0) || r > qreal(1.0)
+        || g < qreal(0.0) || g > qreal(1.0)
+        || b < qreal(0.0) || b > qreal(1.0)
+        || a < qreal(0.0) || a > qreal(1.0)) {
         qWarning("QColor::fromRgbF: RGB parameters out of range");
         return QColor();
     }
@@ -1983,10 +2004,10 @@ QColor QColor::fromHsv(int h, int s, int v, int a)
 */
 QColor QColor::fromHsvF(qreal h, qreal s, qreal v, qreal a)
 {
-    if (((h < 0.0 || h > 1.0) && h != -1.0)
-        || (s < 0.0 || s > 1.0)
-        || (v < 0.0 || v > 1.0)
-        || (a < 0.0 || a > 1.0)) {
+    if (((h < qreal(0.0) || h > qreal(1.0)) && h != qreal(-1.0))
+        || (s < qreal(0.0) || s > qreal(1.0))
+        || (v < qreal(0.0) || v > qreal(1.0))
+        || (a < qreal(0.0) || a > qreal(1.0))) {
         qWarning("QColor::fromHsvF: HSV parameters out of range");
         return QColor();
     }
@@ -1994,7 +2015,7 @@ QColor QColor::fromHsvF(qreal h, qreal s, qreal v, qreal a)
     QColor color;
     color.cspec = Hsv;
     color.ct.ahsv.alpha      = qRound(a * USHRT_MAX);
-    color.ct.ahsv.hue        = h == -1.0 ? USHRT_MAX : qRound(h * 36000);
+    color.ct.ahsv.hue        = h == qreal(-1.0) ? USHRT_MAX : qRound(h * 36000);
     color.ct.ahsv.saturation = qRound(s * USHRT_MAX);
     color.ct.ahsv.value      = qRound(v * USHRT_MAX);
     color.ct.ahsv.pad        = 0;
@@ -2047,10 +2068,10 @@ QColor QColor::fromHsl(int h, int s, int l, int a)
 */
 QColor QColor::fromHslF(qreal h, qreal s, qreal l, qreal a)
 {
-    if (((h < 0.0 || h > 1.0) && h != -1.0)
-        || (s < 0.0 || s > 1.0)
-        || (l < 0.0 || l > 1.0)
-        || (a < 0.0 || a > 1.0)) {
+    if (((h < qreal(0.0) || h > qreal(1.0)) && h != qreal(-1.0))
+        || (s < qreal(0.0) || s > qreal(1.0))
+        || (l < qreal(0.0) || l > qreal(1.0))
+        || (a < qreal(0.0) || a > qreal(1.0))) {
         qWarning("QColor::fromHsvF: HSV parameters out of range");
         return QColor();
     }
@@ -2058,7 +2079,7 @@ QColor QColor::fromHslF(qreal h, qreal s, qreal l, qreal a)
     QColor color;
     color.cspec = Hsl;
     color.ct.ahsl.alpha      = qRound(a * USHRT_MAX);
-    color.ct.ahsl.hue        = (h == -1.0) ? USHRT_MAX : qRound(h * 36000);
+    color.ct.ahsl.hue        = (h == qreal(-1.0)) ? USHRT_MAX : qRound(h * 36000);
     if (color.ct.ahsl.hue == 36000)
         color.ct.ahsl.hue = 0;
     color.ct.ahsl.saturation = qRound(s * USHRT_MAX);
@@ -2167,11 +2188,11 @@ void QColor::setCmyk(int c, int m, int y, int k, int a)
 */
 void QColor::setCmykF(qreal c, qreal m, qreal y, qreal k, qreal a)
 {
-    if (c < 0.0 || c > 1.0
-        || m < 0.0 || m > 1.0
-        || y < 0.0 || y > 1.0
-        || k < 0.0 || k > 1.0
-        || a < 0.0 || a > 1.0) {
+    if (c < qreal(0.0) || c > qreal(1.0)
+        || m < qreal(0.0) || m > qreal(1.0)
+        || y < qreal(0.0) || y > qreal(1.0)
+        || k < qreal(0.0) || k > qreal(1.0)
+        || a < qreal(0.0) || a > qreal(1.0)) {
         qWarning("QColor::setCmykF: CMYK parameters out of range");
         return;
     }
@@ -2229,11 +2250,11 @@ QColor QColor::fromCmyk(int c, int m, int y, int k, int a)
 */
 QColor QColor::fromCmykF(qreal c, qreal m, qreal y, qreal k, qreal a)
 {
-    if (c < 0.0 || c > 1.0
-        || m < 0.0 || m > 1.0
-        || y < 0.0 || y > 1.0
-        || k < 0.0 || k > 1.0
-        || a < 0.0 || a > 1.0) {
+    if (c < qreal(0.0) || c > qreal(1.0)
+        || m < qreal(0.0) || m > qreal(1.0)
+        || y < qreal(0.0) || y > qreal(1.0)
+        || k < qreal(0.0) || k > qreal(1.0)
+        || a < qreal(0.0) || a > qreal(1.0)) {
         qWarning("QColor::fromCmykF: CMYK parameters out of range");
         return QColor();
     }
@@ -2501,7 +2522,7 @@ QDebug operator<<(QDebug dbg, const QColor &c)
 
     Writes the \a color to the \a stream.
 
-    \sa {Format of the QDataStream Operators}
+    \sa {Serializing Qt Data Types}
 */
 QDataStream &operator<<(QDataStream &stream, const QColor &color)
 {
@@ -2537,7 +2558,7 @@ QDataStream &operator<<(QDataStream &stream, const QColor &color)
 
     Reads the \a color from the \a stream.
 
-    \sa { Format of the QDataStream Operators}
+    \sa {Serializing Qt Data Types}
 */
 QDataStream &operator>>(QDataStream &stream, QColor &color)
 {
@@ -2695,12 +2716,4 @@ QDataStream &operator>>(QDataStream &stream, QColor &color)
     \sa QColor::rgb(), QColor::rgba()
 */
 
-/*! \fn void QColormap::initialize()
-  \internal
-*/
-
-/*! \fn void QColormap::cleanup()
-  \internal
-*/
-
 QT_END_NAMESPACE
