@@ -1,8 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -35,6 +34,7 @@
 **
 **
 **
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -66,75 +66,69 @@ QT_BEGIN_NAMESPACE
 #define STATE_CLIP_COMPLEX      0x00020000
 
 
-static inline void updateStateBits(uint *state, uint mask, bool on)
-{
-    *state = on ? (*state | mask) : (*state & ~mask);
-}
-
-static inline bool checkStateAgainstMask(uint state, uint mask)
-{
-    return !state || (state & mask && !(state & ~mask));
-}
-
 class CapabilitiesToStateMask
 {
 public:
     CapabilitiesToStateMask(QBlittable::Capabilities capabilities)
-            : m_capabilities(capabilities),
-              fillRectMask(0),
-              drawRectMask(0),
-              drawPixmapMask(0),
-              capabillitiesState(0)
+        : m_capabilities(capabilities)
+        , fillRectMask(0)
+        , drawRectMask(0)
+        , drawPixmapMask(0)
+        , capabillitiesState(0)
     {
-        if (capabilities & QBlittable::SolidRectCapability) {
+        if (capabilities & QBlittable::SolidRectCapability)
             setFillRectMask();
-        }
-        if (capabilities & QBlittable::SourcePixmapCapability) {
+        if (capabilities & QBlittable::SourcePixmapCapability)
            setSourcePixmapMask();
-        }
-        if (capabilities & QBlittable::SourceOverPixmapCapability) {
+        if (capabilities & QBlittable::SourceOverPixmapCapability)
            setSourceOverPixmapMask();
-        }
-        if (capabilities & QBlittable::SourceOverScaledPixmapCapability) {
+        if (capabilities & QBlittable::SourceOverScaledPixmapCapability)
             setSourceOverScaledPixmapMask();
-        }
     }
 
     inline bool canBlitterFillRect() const
     {
-        return checkStateAgainstMask(capabillitiesState,fillRectMask);
+        return checkStateAgainstMask(capabillitiesState, fillRectMask);
     }
 
     inline bool canBlitterDrawRectMask() const
     {
-        return checkStateAgainstMask(capabillitiesState,drawRectMask);
+        return checkStateAgainstMask(capabillitiesState, drawRectMask);
     }
 
     bool canBlitterDrawPixmap(const QRectF &r, const QPixmap &pm, const QRectF &sr) const
     {
         if (pm.pixmapData()->classId() != QPixmapData::BlitterClass)
             return false;
-        if (checkStateAgainstMask(capabillitiesState,drawPixmapMask)) {
+        if (checkStateAgainstMask(capabillitiesState, drawPixmapMask)) {
             if (m_capabilities & (QBlittable::SourceOverPixmapCapability
                                   | QBlittable::SourceOverScaledPixmapCapability)) {
-                if (r.size() != sr.size()) {
+                if (r.size() != sr.size())
                     return m_capabilities & QBlittable::SourceOverScaledPixmapCapability;
-                } else {
+                else
                     return m_capabilities & QBlittable::SourceOverPixmapCapability;
-                }
             }
-            if ((m_capabilities & QBlittable::SourcePixmapCapability) && r.size() == sr.size() && !pm.hasAlphaChannel()) {
+            if ((m_capabilities & QBlittable::SourcePixmapCapability) && r.size() == sr.size() && !pm.hasAlphaChannel())
                 return m_capabilities & QBlittable::SourcePixmapCapability;
-            }
         }
         return false;
     }
 
     inline void updateState(uint mask, bool on) {
-        updateStateBits(&capabillitiesState,mask,on);
+        updateStateBits(&capabillitiesState, mask, on);
     }
 
-public:
+private:
+
+    static inline void updateStateBits(uint *state, uint mask, bool on)
+    {
+        *state = on ? (*state | mask) : (*state & ~mask);
+    }
+
+    static inline bool checkStateAgainstMask(uint state, uint mask)
+    {
+        return !state || (state & mask && !(state & ~mask));
+    }
 
     void setFillRectMask() {
         updateStateBits(&fillRectMask, STATE_XFORM_SCALE, false);
@@ -187,148 +181,266 @@ public:
     uint capabillitiesState;
 };
 
-class QBlitterPaintEnginePrivate : public QPaintEngineExPrivate
+class QBlitterPaintEnginePrivate : public QRasterPaintEnginePrivate
 {
     Q_DECLARE_PUBLIC(QBlitterPaintEngine);
 public:
     QBlitterPaintEnginePrivate(QBlittablePixmapData *p)
-            : QPaintEngineExPrivate(),
-              pmData(p),
-              isBlitterLocked(false),
-              hasXForm(false)
+        : QRasterPaintEnginePrivate()
+        , pmData(p)
+        , caps(pmData->blittable()->capabilities())
+        , hasXForm(false)
 
-    {
-        raster = new QRasterPaintEngine(p->buffer());
-        capabillities = new CapabilitiesToStateMask(pmData->blittable()->capabilities());
-    }
+    {}
 
-    inline void lock() {
-        if (!isBlitterLocked) {
-            raster->d_func()->rasterBuffer->prepare(pmData->blittable()->lock());
-            isBlitterLocked = true;
-        }
-    }
+    void lock();
+    void unlock();
+    void fillRect(const QRectF& rect, const QColor&);
+    void clipAndDrawPixmap(const QRectF &clip, const QRectF &target, const QPixmap &pm, const QRectF &sr);
 
-    inline void unlock() {
-        if (isBlitterLocked) {
-            pmData->blittable()->unlock();
-            isBlitterLocked = false;
-        }
-    }
 
-    void fillRect(const QRectF &rect, const QColor &color) {
-        Q_Q(QBlitterPaintEngine);
-        pmData->unmarkRasterOverlay(rect);
-        QRectF targetRect = rect;
-        if (hasXForm) {
-            targetRect = q->state()->matrix.mapRect(rect);
-        }
-        const QClipData *clipData = q->clip();
-        if (clipData) {
-            if (clipData->hasRectClip) {
-                unlock();
-                pmData->blittable()->fillRect(targetRect & clipData->clipRect, color);
-            } else if (clipData->hasRegionClip) {
-                QVector<QRect> rects = clipData->clipRegion.rects();
-                for ( int i = 0; i < rects.size(); i++ ) {
-                    QRect intersectRect = rects.at(i).intersected(targetRect.toRect());
-                    if (!intersectRect.isEmpty()) {
-                        unlock();
-                        pmData->blittable()->fillRect(intersectRect,color);
-                    }
+    void updateCompleteState(QPainterState *s);
+    void updatePenState(QPainterState *s);
+    void updateBrushState(QPainterState *s);
+    void updateOpacityState(QPainterState *s);
+    void updateCompositionModeState(QPainterState *s);
+    void updateRenderHintsState(QPainterState *s);
+    void updateTransformState(QPainterState *s);
+    void updateClipState(QPainterState *s);
+
+    QBlittablePixmapData *pmData;
+    CapabilitiesToStateMask caps;
+    uint hasXForm;
+};
+
+inline void QBlitterPaintEnginePrivate::lock()
+{
+    if (!pmData->blittable()->isLocked())
+        rasterBuffer->prepare(pmData->buffer());
+}
+
+inline void QBlitterPaintEnginePrivate::unlock()
+{
+    pmData->blittable()->unlock();
+}
+
+// State tracking to make decisions
+void QBlitterPaintEnginePrivate::updateCompleteState(QPainterState *s)
+{
+    updatePenState(s);
+    updateBrushState(s);
+    updateOpacityState(s);
+    updateCompositionModeState(s);
+    updateRenderHintsState(s);
+    updateTransformState(s);
+    updateClipState(s);
+}
+
+void QBlitterPaintEnginePrivate::updatePenState(QPainterState *s)
+{
+    caps.updateState(STATE_PEN_ENABLED, qpen_style(s->pen) != Qt::NoPen);
+}
+
+void QBlitterPaintEnginePrivate::updateBrushState(QPainterState *s)
+{
+    Qt::BrushStyle style = qbrush_style(s->brush);
+
+    caps.updateState(STATE_BRUSH_PATTERN, style > Qt::SolidPattern);
+    caps.updateState(STATE_BRUSH_ALPHA,
+                        qbrush_color(s->brush).alpha() < 255);
+}
+
+void QBlitterPaintEnginePrivate::updateOpacityState(QPainterState *s)
+{
+    bool translucent = s->opacity < 1;
+    caps.updateState(STATE_ALPHA, translucent);
+}
+
+void QBlitterPaintEnginePrivate::updateCompositionModeState(QPainterState *s)
+{
+    bool nonTrivial = s->composition_mode != QPainter::CompositionMode_SourceOver
+                      && s->composition_mode != QPainter::CompositionMode_Source;
+
+    caps.updateState(STATE_BLENDING_COMPLEX, nonTrivial);
+}
+
+void QBlitterPaintEnginePrivate::updateRenderHintsState(QPainterState *s)
+{
+    bool aa = s->renderHints & QPainter::Antialiasing;
+    caps.updateState(STATE_ANTIALIASING, aa);
+}
+
+void QBlitterPaintEnginePrivate::updateTransformState(QPainterState *s)
+{
+    QTransform::TransformationType type = s->matrix.type();
+
+    caps.updateState(STATE_XFORM_COMPLEX, type > QTransform::TxScale);
+    caps.updateState(STATE_XFORM_SCALE, type > QTransform::TxTranslate);
+
+    hasXForm = type >= QTransform::TxTranslate;
+}
+
+void QBlitterPaintEnginePrivate::updateClipState(QPainterState *)
+{
+    const QClipData *clipData = clip();
+    bool complexClip = clipData && !(clipData->hasRectClip || clipData->hasRegionClip);
+    caps.updateState(STATE_CLIP_COMPLEX, complexClip);
+}
+
+void QBlitterPaintEnginePrivate::fillRect(const QRectF &rect, const QColor &color)
+{
+    Q_Q(QBlitterPaintEngine);
+    pmData->unmarkRasterOverlay(rect);
+    QRectF targetRect = rect;
+    if (hasXForm)
+        targetRect = q->state()->matrix.mapRect(rect);
+    const QClipData *clipData = clip();
+    if (clipData) {
+        if (clipData->hasRectClip) {
+            unlock();
+            pmData->blittable()->fillRect(targetRect & clipData->clipRect, color);
+        } else if (clipData->hasRegionClip) {
+            QVector<QRect> rects = clipData->clipRegion.rects();
+            for (int i = 0; i < rects.size(); ++i) {
+                QRect intersectRect = rects.at(i).intersected(targetRect.toRect());
+                if (!intersectRect.isEmpty()) {
+                    unlock();
+                    pmData->blittable()->fillRect(intersectRect, color);
                 }
             }
+        }
+    } else {
+        if (targetRect.x() >= 0 && targetRect.y() >= 0
+            && targetRect.width() <= q->paintDevice()->width()
+            && targetRect.height() <= q->paintDevice()->height()) {
+            unlock();
+            pmData->blittable()->fillRect(targetRect, color);
         } else {
-            if (targetRect.x() >= 0 && targetRect.y() >= 0
-                && targetRect.width() <= raster->paintDevice()->width()
-                && targetRect.height() <= raster->paintDevice()->height()) {
-                unlock();
-                pmData->blittable()->fillRect(targetRect,color);
-            } else {
-                QRectF deviceRect(0,0,raster->paintDevice()->width(), raster->paintDevice()->height());
-                unlock();
-                pmData->blittable()->fillRect(deviceRect&targetRect,color);
-            }
+            QRectF deviceRect(0, 0, q->paintDevice()->width(), q->paintDevice()->height());
+            unlock();
+            pmData->blittable()->fillRect(deviceRect & targetRect, color);
         }
     }
+}
 
-    void clipAndDrawPixmap(const QRectF &clip, const QRectF &target, const QPixmap &pm, const QRectF &sr) {
-        QRectF intersectedRect = clip.intersected(target);
-        if (intersectedRect.isEmpty())
-            return;
-        QRectF source = sr;
-        if(intersectedRect.size() != target.size()) {
+void QBlitterPaintEnginePrivate::clipAndDrawPixmap(const QRectF &clip, const QRectF &target, const QPixmap &pm, const QRectF &sr)
+{
+    QRectF intersectedRect = clip.intersected(target);
+    if (intersectedRect.isEmpty())
+        return;
+    QRectF source = sr;
+    if (intersectedRect.size() != target.size()) {
+        if (sr.size() == target.size()) {
+            // no resize
             qreal deltaTop = target.top() - intersectedRect.top();
             qreal deltaLeft = target.left() - intersectedRect.left();
             qreal deltaBottom = target.bottom() - intersectedRect.bottom();
             qreal deltaRight = target.right() - intersectedRect.right();
-            source.adjust(-deltaLeft,-deltaTop,-deltaRight,-deltaBottom);
+            source.adjust(-deltaLeft, -deltaTop, -deltaRight, -deltaBottom);
+        } else {
+            // resize case
+            qreal hFactor = sr.size().width() / target.size().width();
+            qreal vFactor = sr.size().height() / target.size().height();
+            qreal deltaTop = (target.top() - intersectedRect.top()) * vFactor;
+            qreal deltaLeft = (target.left() - intersectedRect.left()) * hFactor;
+            qreal deltaBottom = (target.bottom() - intersectedRect.bottom()) * vFactor;
+            qreal deltaRight = (target.right() - intersectedRect.right()) * hFactor;
+            source.adjust(-deltaLeft, -deltaTop, -deltaRight, -deltaBottom);
         }
-        pmData->unmarkRasterOverlay(intersectedRect);
-        pmData->blittable()->drawPixmap(intersectedRect, pm, source);
     }
-
-    void updateClip() {
-        Q_Q(QBlitterPaintEngine);
-        const QClipData *clip = q->clip();
-        bool complex = clip && !(clip->hasRectClip || clip->hasRegionClip);
-        capabillities->updateState(STATE_CLIP_COMPLEX, complex);
-    }
-
-    void systemStateChanged() {
-        raster->d_func()->systemStateChanged();
-    }
-
-    QRasterPaintEngine *raster;
-
-    QBlittablePixmapData *pmData;
-    bool isBlitterLocked;
-
-    CapabilitiesToStateMask *capabillities;
-
-    uint hasXForm;
-};
+    pmData->unmarkRasterOverlay(intersectedRect);
+    pmData->blittable()->drawPixmap(intersectedRect, pm, source);
+}
 
 QBlitterPaintEngine::QBlitterPaintEngine(QBlittablePixmapData *p)
-    : QPaintEngineEx(*(new QBlitterPaintEnginePrivate(p)))
+    : QRasterPaintEngine(*(new QBlitterPaintEnginePrivate(p)), p->buffer())
+{}
+
+// State tracking
+void QBlitterPaintEngine::penChanged()
 {
+    Q_D(QBlitterPaintEngine);
+
+    QRasterPaintEngine::penChanged();
+    d->updatePenState(state());
 }
 
-QBlitterPaintEngine::~QBlitterPaintEngine()
+void QBlitterPaintEngine::brushChanged()
 {
+    Q_D(QBlitterPaintEngine);
+
+    QRasterPaintEngine::brushChanged();
+    d->updateBrushState(state());
 }
 
-QPainterState *QBlitterPaintEngine::createState(QPainterState *orig) const
+void QBlitterPaintEngine::opacityChanged()
 {
-    Q_D(const QBlitterPaintEngine);
-    return d->raster->createState(orig);
+    Q_D(QBlitterPaintEngine);
+
+    QRasterPaintEngine::opacityChanged();
+    d->updateOpacityState(state());
+}
+
+void QBlitterPaintEngine::compositionModeChanged()
+{
+    Q_D(QBlitterPaintEngine);
+
+    QRasterPaintEngine::compositionModeChanged();
+    d->updateCompositionModeState(state());
+}
+
+void QBlitterPaintEngine::renderHintsChanged()
+{
+    Q_D(QBlitterPaintEngine);
+
+    QRasterPaintEngine::renderHintsChanged();
+    d->updateRenderHintsState(state());
+}
+
+void QBlitterPaintEngine::transformChanged()
+{
+    Q_D(QBlitterPaintEngine);
+
+    QRasterPaintEngine::transformChanged();
+    d->updateTransformState(state());
+}
+
+void QBlitterPaintEngine::clipEnabledChanged()
+{
+    Q_D(QBlitterPaintEngine);
+    QRasterPaintEngine::clipEnabledChanged();
+    d->updateClipState(state());
 }
 
 bool QBlitterPaintEngine::begin(QPaintDevice *pdev)
 {
-    Q_D(QBlitterPaintEngine);
-
-    setActive(true);
-    bool ok = d->raster->begin(pdev);
+    bool ok = QRasterPaintEngine::begin(pdev);
 #ifdef QT_BLITTER_RASTEROVERLAY
+    Q_D(QBlitterPaintEngine);
     d->pmData->unmergeOverlay();
 #endif
     return ok;
 }
 
-
 bool QBlitterPaintEngine::end()
+{
+#ifdef QT_BLITTER_RASTEROVERLAY
+    Q_D(QBlitterPaintEngine);
+    d->pmData->mergeOverlay();
+#endif
+
+    return QRasterPaintEngine::end();
+}
+
+void QBlitterPaintEngine::setState(QPainterState *s)
 {
     Q_D(QBlitterPaintEngine);
 
-    setActive(false);
-#ifdef QT_BLITTER_RASTEROVERLAY
-    d->pmData->mergeOverlay();
-#endif
-    return d->raster->end();
+    QRasterPaintEngine::setState(s);
+    d->updateCompleteState(s);
 }
 
-
+// Accelerated paths
 void QBlitterPaintEngine::fill(const QVectorPath &path, const QBrush &brush)
 {
     Q_D(QBlitterPaintEngine);
@@ -338,37 +450,35 @@ void QBlitterPaintEngine::fill(const QVectorPath &path, const QBrush &brush)
     } else {
         d->lock();
         d->pmData->markRasterOverlay(path);
-        d->raster->fill(path, brush);
+        QRasterPaintEngine::fill(path, brush);
     }
 }
 
 void QBlitterPaintEngine::fillRect(const QRectF &rect, const QColor &color)
 {
     Q_D(QBlitterPaintEngine);
-    if (d->capabillities->canBlitterFillRect() && color.alpha() == 0xff) {
+    if (d->caps.canBlitterFillRect() && color.alpha() == 0xff) {
         d->fillRect(rect, color);
     } else {
         d->lock();
         d->pmData->markRasterOverlay(rect);
-        d->raster->fillRect(rect, color);
+        QRasterPaintEngine::fillRect(rect, color);
     }
 }
 
 void QBlitterPaintEngine::fillRect(const QRectF &rect, const QBrush &brush)
 {
-    if(rect.size().isEmpty())
+    if (rect.size().isEmpty())
         return;
 
     Q_D(QBlitterPaintEngine);
 
     if (qbrush_style(brush) == Qt::SolidPattern
         && qbrush_color(brush).alpha() == 0xff
-        && d->capabillities->canBlitterFillRect())
-    {
+        && d->caps.canBlitterFillRect()) {
         d->fillRect(rect, qbrush_color(brush));
-    }else if (brush.style() == Qt::TexturePattern
-              && d->capabillities->canBlitterDrawPixmap(rect,brush.texture(),rect))
-    {
+    } else if (brush.style() == Qt::TexturePattern
+              && d->caps.canBlitterDrawPixmap(rect, brush.texture(), rect)) {
         bool rectIsFilled = false;
         QRectF transformedRect = state()->matrix.mapRect(rect);
         qreal x = transformedRect.x();
@@ -389,37 +499,37 @@ void QBlitterPaintEngine::fillRect(const QRectF &rect, const QBrush &brush)
                 blitWidth = transformedRect.right() -x;
             if (y + blitHeight > transformedRect.bottom())
                 blitHeight = transformedRect.bottom() - y;
-            const QClipData *clipData = clip();
+            const QClipData *clipData = d->clip();
             if (clipData->hasRectClip) {
-                QRect targetRect = QRect(x,y,blitWidth,blitHeight).intersected(clipData->clipRect);
+                QRect targetRect = QRect(x, y, blitWidth, blitHeight).intersected(clipData->clipRect);
                 if (targetRect.isValid()) {
                     int tmpSrcX  = srcX + (targetRect.x() - x);
                     int tmpSrcY = srcY + (targetRect.y() - y);
-                    QRect srcRect(tmpSrcX,tmpSrcY,targetRect.width(),targetRect.height());
-                    d->pmData->blittable()->drawPixmap(targetRect,pm,srcRect);
+                    QRect srcRect(tmpSrcX, tmpSrcY, targetRect.width(), targetRect.height());
+                    d->pmData->blittable()->drawPixmap(targetRect, pm, srcRect);
                 }
             } else if (clipData->hasRegionClip) {
                 QVector<QRect> clipRects = clipData->clipRegion.rects();
-                QRect unclippedTargetRect(x,y,blitWidth,blitHeight);
+                QRect unclippedTargetRect(x, y, blitWidth, blitHeight);
                 QRegion intersectedRects = clipData->clipRegion.intersected(unclippedTargetRect);
 
-                for ( int i = 0; i < intersectedRects.rects().size(); i++ ) {
+                for (int i = 0; i < intersectedRects.rects().size(); ++i) {
                     QRect targetRect = intersectedRects.rects().at(i);
                     if (!targetRect.isValid() || targetRect.isEmpty())
                         continue;
                     int tmpSrcX = srcX + (targetRect.x() - x);
                     int tmpSrcY = srcY + (targetRect.y() - y);
-                    QRect srcRect(tmpSrcX,tmpSrcY,targetRect.width(),targetRect.height());
-                    d->pmData->blittable()->drawPixmap(targetRect,pm,srcRect);
+                    QRect srcRect(tmpSrcX, tmpSrcY, targetRect.width(), targetRect.height());
+                    d->pmData->blittable()->drawPixmap(targetRect, pm, srcRect);
                 }
             }
             x+=blitWidth;
-            if (x>=transformedRect.right()) {
+            if (qFuzzyCompare(x, transformedRect.right())) {
                 x = transformedRect.x();
                 srcX = startX;
                 srcY = 0;
-                y+=blitHeight;
-                if (y>=transformedRect.bottom())
+                y += blitHeight;
+                if (qFuzzyCompare(y, transformedRect.bottom()))
                     rectIsFilled = true;
             } else
                 srcX = 0;
@@ -427,235 +537,174 @@ void QBlitterPaintEngine::fillRect(const QRectF &rect, const QBrush &brush)
     } else {
         d->lock();
         d->pmData->markRasterOverlay(rect);
-        d->raster->fillRect(rect, brush);
+        QRasterPaintEngine::fillRect(rect, brush);
     }
 
 }
 
-void QBlitterPaintEngine::stroke(const QVectorPath &path, const QPen &pen)
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->pmData->markRasterOverlay(path);
-    d->raster->stroke(path, pen);
-}
-
-void QBlitterPaintEngine::clip(const QVectorPath &path, Qt::ClipOperation op)
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->raster->clip(path, op);
-    d->updateClip();
-}
-void QBlitterPaintEngine::clip(const QRect &rect, Qt::ClipOperation op){
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->raster->clip(rect,op);
-    d->updateClip();
-}
-void QBlitterPaintEngine::clip(const QRegion &region, Qt::ClipOperation op)
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->raster->clip(region,op);
-    d->updateClip();
-}
-
-void QBlitterPaintEngine::clipEnabledChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->raster->clipEnabledChanged();
-}
-
-void QBlitterPaintEngine::penChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->raster->penChanged();
-    d->capabillities->updateState(STATE_PEN_ENABLED,qpen_style(state()->pen) != Qt::NoPen);
-}
-
-void QBlitterPaintEngine::brushChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->raster->brushChanged();
-
-    bool solid = qbrush_style(state()->brush) == Qt::SolidPattern;
-
-    d->capabillities->updateState(STATE_BRUSH_PATTERN, !solid);
-    d->capabillities->updateState(STATE_BRUSH_ALPHA,
-                                        qbrush_color(state()->brush).alpha() < 255);
-}
-
-void QBlitterPaintEngine::brushOriginChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->raster->brushOriginChanged();
-}
-
-void QBlitterPaintEngine::opacityChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->raster->opacityChanged();
-
-    bool translucent = state()->opacity < 1;
-    d->capabillities->updateState(STATE_ALPHA,translucent);
-}
-
-void QBlitterPaintEngine::compositionModeChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->raster->compositionModeChanged();
-
-    bool nonTrivial = state()->composition_mode != QPainter::CompositionMode_SourceOver
-                      && state()->composition_mode != QPainter::CompositionMode_Source;
-
-    d->capabillities->updateState(STATE_BLENDING_COMPLEX,nonTrivial);
-}
-
-void QBlitterPaintEngine::renderHintsChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->raster->renderHintsChanged();
-
-    bool aa = state()->renderHints & QPainter::Antialiasing;
-    d->capabillities->updateState(STATE_ANTIALIASING, aa);
-
-}
-
-void QBlitterPaintEngine::transformChanged()
-{
-    Q_D(QBlitterPaintEngine);
-    d->raster->transformChanged();
-
-    QTransform::TransformationType type = state()->matrix.type();
-
-    d->capabillities->updateState(STATE_XFORM_COMPLEX, type > QTransform::TxScale);
-    d->capabillities->updateState(STATE_XFORM_SCALE, type > QTransform::TxTranslate);
-
-    d->hasXForm = type >= QTransform::TxTranslate;
-
-}
-
 void QBlitterPaintEngine::drawRects(const QRect *rects, int rectCount)
 {
     Q_D(QBlitterPaintEngine);
-    if (d->capabillities->canBlitterDrawRectMask()) {
-        for (int i=0; i<rectCount; ++i) {
+    if (d->caps.canBlitterDrawRectMask()) {
+        for (int i=0; i<rectCount; ++i)
             d->fillRect(rects[i], qbrush_color(state()->brush));
-        }
     } else {
-        d->pmData->markRasterOverlay(rects,rectCount);
-        QPaintEngineEx::drawRects(rects, rectCount);
+        d->pmData->markRasterOverlay(rects, rectCount);
+        QRasterPaintEngine::drawRects(rects, rectCount);
     }
 }
 
 void QBlitterPaintEngine::drawRects(const QRectF *rects, int rectCount)
 {
     Q_D(QBlitterPaintEngine);
-    if (d->capabillities->canBlitterDrawRectMask()) {
-        for (int i=0; i<rectCount; ++i) {
+    if (d->caps.canBlitterDrawRectMask()) {
+        for (int i = 0; i < rectCount; ++i)
             d->fillRect(rects[i], qbrush_color(state()->brush));
-        }
     } else {
-        d->pmData->markRasterOverlay(rects,rectCount);
-        QPaintEngineEx::drawRects(rects, rectCount);
+        d->pmData->markRasterOverlay(rects, rectCount);
+        QRasterPaintEngine::drawRects(rects, rectCount);
     }
 }
 
+void QBlitterPaintEngine::drawPixmap(const QPointF &pos, const QPixmap &pm)
+{
+    drawPixmap(QRectF(pos, pm.size()), pm, pm.rect());
+}
+
 void QBlitterPaintEngine::drawPixmap(const QRectF &r, const QPixmap &pm, const QRectF &sr)
 {
     Q_D(QBlitterPaintEngine);
-    if (d->capabillities->canBlitterDrawPixmap(r,pm,sr)) {
+    if (d->caps.canBlitterDrawPixmap(r, pm, sr)) {
 
         d->unlock();
         QRectF targetRect = r;
-        if (d->hasXForm) {
+        if (d->hasXForm)
             targetRect = state()->matrix.mapRect(r);
-        }
-        const QClipData *clipData = clip();
+        const QClipData *clipData = d->clip();
         if (clipData) {
             if (clipData->hasRectClip) {
-                d->clipAndDrawPixmap(clipData->clipRect,targetRect,pm,sr);
-            }else if (clipData->hasRegionClip) {
+                d->clipAndDrawPixmap(clipData->clipRect, targetRect, pm, sr);
+            } else if (clipData->hasRegionClip) {
                 QVector<QRect>rects = clipData->clipRegion.rects();
-                for (int i = 0; i<rects.size(); i++) {
-                    d->clipAndDrawPixmap(rects.at(i),targetRect,pm,sr);
-                }
+                for (int i = 0; i<rects.size(); i++)
+                    d->clipAndDrawPixmap(rects.at(i), targetRect, pm, sr);
             }
         } else {
-            QRectF deviceRect(0,0,d->raster->paintDevice()->width(), d->raster->paintDevice()->height());
-            d->clipAndDrawPixmap(deviceRect,targetRect,pm,sr);
+            QRectF deviceRect(0, 0, paintDevice()->width(), paintDevice()->height());
+            d->clipAndDrawPixmap(deviceRect, targetRect, pm, sr);
         }
     }else {
         d->lock();
         d->pmData->markRasterOverlay(r);
-        d->raster->drawPixmap(r, pm, sr);
+        QRasterPaintEngine::drawPixmap(r, pm, sr);
     }
 }
 
+// Overriden methods to lock the graphics memory
+void QBlitterPaintEngine::drawPolygon(const QPointF *points, int pointCount, PolygonDrawMode mode)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(points, pointCount);
+    QRasterPaintEngine::drawPolygon(points, pointCount, mode);
+}
+
+void QBlitterPaintEngine::drawPolygon(const QPoint *points, int pointCount, PolygonDrawMode mode)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(points, pointCount);
+    QRasterPaintEngine::drawPolygon(points, pointCount, mode);
+}
+
+void QBlitterPaintEngine::fillPath(const QPainterPath &path, QSpanData *fillData)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(path);
+    QRasterPaintEngine::fillPath(path, fillData);
+}
+
+void QBlitterPaintEngine::fillPolygon(const QPointF *points, int pointCount, PolygonDrawMode mode)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(points, pointCount);
+    QRasterPaintEngine::fillPolygon(points, pointCount, mode);
+}
+
+void QBlitterPaintEngine::drawEllipse(const QRectF &r)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(r);
+    QRasterPaintEngine::drawEllipse(r);
+}
+
+void QBlitterPaintEngine::drawImage(const QPointF &pos, const QImage &image)
+{
+    drawImage(QRectF(pos, image.size()), image, image.rect());
+}
+
 void QBlitterPaintEngine::drawImage(const QRectF &r, const QImage &pm, const QRectF &sr,
                                     Qt::ImageConversionFlags flags)
 {
     Q_D(QBlitterPaintEngine);
     d->lock();
     d->pmData->markRasterOverlay(r);
-    d->raster->drawImage(r, pm, sr, flags);
+    QRasterPaintEngine::drawImage(r, pm, sr, flags);
 }
 
+void QBlitterPaintEngine::drawTiledPixmap(const QRectF &r, const QPixmap &pm, const QPointF &sr)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(r);
+    QRasterPaintEngine::drawTiledPixmap(r, pm, sr);
+}
 
 void QBlitterPaintEngine::drawTextItem(const QPointF &pos, const QTextItem &ti)
 {
     Q_D(QBlitterPaintEngine);
     d->lock();
-    d->raster->drawTextItem(pos, ti);
-    d->pmData->markRasterOverlay(pos,ti);
+    d->pmData->markRasterOverlay(pos, ti);
+    QRasterPaintEngine::drawTextItem(pos, ti);
+}
+
+void QBlitterPaintEngine::drawPoints(const QPointF *points, int pointCount)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(points, pointCount);
+    QRasterPaintEngine::drawPoints(points, pointCount);
+}
+
+void QBlitterPaintEngine::drawPoints(const QPoint *points, int pointCount)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(points, pointCount);
+    QRasterPaintEngine::drawPoints(points, pointCount);
+}
+
+void QBlitterPaintEngine::stroke(const QVectorPath &path, const QPen &pen)
+{
+    Q_D(QBlitterPaintEngine);
+    d->lock();
+    d->pmData->markRasterOverlay(path);
+    QRasterPaintEngine::stroke(path, pen);
 }
 
 void QBlitterPaintEngine::drawStaticTextItem(QStaticTextItem *sti)
 {
     Q_D(QBlitterPaintEngine);
     d->lock();
-    d->raster->drawStaticTextItem(sti);
+    QRasterPaintEngine::drawStaticTextItem(sti);
 
+#ifdef QT_BLITTER_RASTEROVERLAY
 //#### d->pmData->markRasterOverlay(sti);
     qWarning("not implemented: markRasterOverlay for QStaticTextItem");
-
-}
-
-
-void QBlitterPaintEngine::drawEllipse(const QRectF &r)
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    d->pmData->markRasterOverlay(r);
-    d->raster->drawEllipse(r);
-}
-
-void QBlitterPaintEngine::setState(QPainterState *s)
-{
-    Q_D(QBlitterPaintEngine);
-    d->lock();
-    QPaintEngineEx::setState(s);
-    d->raster->setState(s);
-
-    clipEnabledChanged();
-    penChanged();
-    brushChanged();
-    brushOriginChanged();
-    opacityChanged();
-    compositionModeChanged();
-    renderHintsChanged();
-    transformChanged();
-
-    d->updateClip();
-}
-
-inline QRasterPaintEngine *QBlitterPaintEngine::raster() const
-{
-    Q_D(const QBlitterPaintEngine);
-    return d->raster;
+#endif
 }
 
 QT_END_NAMESPACE
