@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -251,8 +251,8 @@ QList<QByteArray> QFontEngineQPF::cleanUpAfterClientCrash(const QList<int> &cras
 {
     QList<QByteArray> removedFonts;
     QDir dir(qws_fontCacheDir(), QLatin1String("*.qsf"));
-    for (int i = 0; i < int(dir.count()); ++i) {
-        const QByteArray fileName = QFile::encodeName(dir.absoluteFilePath(dir[i]));
+    foreach (const QFileInfo &fi, dir.entryInfoList()) {
+        const QByteArray fileName = QFile::encodeName(fi.absoluteFilePath());
 
         int fd = QT_OPEN(fileName.constData(), O_RDONLY, 0);
         if (fd >= 0) {
@@ -278,15 +278,12 @@ QList<QByteArray> QFontEngineQPF::cleanUpAfterClientCrash(const QList<int> &cras
 
 static inline unsigned int getChar(const QChar *str, int &i, const int len)
 {
-    unsigned int uc = str[i].unicode();
-    if (uc >= 0xd800 && uc < 0xdc00 && i < len-1) {
-        uint low = str[i+1].unicode();
-       if (low >= 0xdc00 && low < 0xe000) {
-            uc = (uc - 0xd800)*0x400 + (low - 0xdc00) + 0x10000;
-            ++i;
-        }
+    uint ucs4 = str[i].unicode();
+    if (str[i].isHighSurrogate() && i < len-1 && str[i+1].isLowSurrogate()) {
+        ++i;
+        ucs4 = QChar::surrogateToUcs4(ucs4, str[i].unicode());
     }
-    return uc;
+    return ucs4;
 }
 #ifdef QT_FONTS_ARE_RESOURCES
 QFontEngineQPF::QFontEngineQPF(const QFontDef &def, const uchar *bytes, int size)
@@ -306,6 +303,8 @@ QFontEngineQPF::QFontEngineQPF(const QFontDef &def, int fileDescriptor, QFontEng
     glyphMapEntries = 0;
     glyphDataOffset = 0;
     glyphDataSize = 0;
+    if (renderingFontEngine)
+        glyphFormat = renderingFontEngine->glyphFormat;
     kerning_pairs_loaded = false;
     readOnly = true;
 
@@ -340,14 +339,14 @@ QFontEngineQPF::QFontEngineQPF(const QFontDef &def, int fileDescriptor, QFontEng
                 fd = QT_OPEN(encodedFileName, O_RDONLY);
                 if (fd == -1) {
 #if defined(DEBUG_FONTENGINE)
-                    qErrnoWarning("QFontEngineQPF: unable to open %s", encodedName.constData());
+                    qErrnoWarning("QFontEngineQPF: unable to open %s", encodedFileName.constData());
 #endif
                     return;
                 }
             }
             if (fd == -1) {
 #if defined(DEBUG_FONTENGINE)
-                qWarning("QFontEngineQPF: insufficient access rights to %s", encodedName.constData());
+                qWarning("QFontEngineQPF: insufficient access rights to %s", encodedFileName.constData());
 #endif
                 return;
             }
@@ -359,7 +358,7 @@ QFontEngineQPF::QFontEngineQPF(const QFontDef &def, int fileDescriptor, QFontEng
                 fd = QT_OPEN(encodedFileName, O_RDWR | O_EXCL | O_CREAT, 0644);
                 if (fd == -1) {
 #if defined(DEBUG_FONTENGINE)
-                    qErrnoWarning(errno, "QFontEngineQPF: open() failed for %s", encodedName.constData());
+                    qErrnoWarning(errno, "QFontEngineQPF: open() failed for %s", encodedFileName.constData());
 #endif
                     return;
                 }
@@ -372,7 +371,7 @@ QFontEngineQPF::QFontEngineQPF(const QFontDef &def, int fileDescriptor, QFontEng
                 const QByteArray &data = buffer.data();
                 if (QT_WRITE(fd, data.constData(), data.size()) == -1) {
 #if defined(DEBUG_FONTENGINE)
-                    qErrnoWarning(errno, "QFontEngineQPF: write() failed for %s", encodedName.constData());
+                    qErrnoWarning(errno, "QFontEngineQPF: write() failed for %s", encodedFileName.constData());
 #endif
                     return;
                 }
@@ -578,8 +577,6 @@ bool QFontEngineQPF::stringToCMap(const QChar *str, int len, QGlyphLayout *glyph
     if (symbol) {
         for (int i = 0; i < len; ++i) {
             unsigned int uc = getChar(str, i, len);
-            if (mirrored)
-                uc = QChar::mirroredChar(uc);
             glyphs->glyphs[glyph_pos] = getTrueTypeGlyphIndex(cmap, uc);
             if(!glyphs->glyphs[glyph_pos] && uc < 0x100)
                 glyphs->glyphs[glyph_pos] = getTrueTypeGlyphIndex(cmap, uc + 0xf000);
@@ -920,8 +917,18 @@ void QFontEngineQPF::loadGlyph(glyph_t glyph)
 
     if (!renderingFontEngine)
         return;
-
-    QImage img = renderingFontEngine->alphaMapForGlyph(glyph).convertToFormat(QImage::Format_Indexed8);
+    QImage img = renderingFontEngine->alphaMapForGlyph(glyph);
+    if (img.format() != QImage::Format_Indexed8) {
+        bool mono = img.depth() == 1;
+        img = img.convertToFormat(QImage::Format_Indexed8);
+        if (mono) {
+            //### we know that 1 is opaque and 0 is transparent
+            uchar *byte = img.bits();
+            int count = img.byteCount();
+            while (count--)
+                *byte++ *= 0xff;
+        }
+    }
     glyph_metrics_t metrics = renderingFontEngine->boundingBox(glyph);
     renderingFontEngine->removeGlyphFromCache(glyph);
 
