@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -754,19 +754,59 @@ void QTextTable::insertColumns(int pos, int num)
     QTextFormatCollection *c = p->formatCollection();
     p->beginEditBlock();
 
+    QList<int> extendedSpans;
     for (int i = 0; i < d->nRows; ++i) {
         int cell;
-        if (i == d->nRows - 1 && pos == d->nCols)
+        if (i == d->nRows - 1 && pos == d->nCols) {
             cell = d->fragment_end;
-        else
-            cell = d->grid[i*d->nCols + pos];
-        QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
-        QTextCharFormat fmt = c->charFormat(it->format);
+        } else {
+            int logicalGridIndexBeforePosition = pos > 0
+                                                 ? d->findCellIndex(d->grid[i*d->nCols + pos - 1])
+                                                 : -1;
+
+            // Search for the logical insertion point by skipping past cells which are not the first
+            // cell in a rowspan. This means any cell for which the logical grid index is
+            // less than the logical cell index of the cell before the insertion.
+            int logicalGridIndex;
+            int gridArrayOffset = i*d->nCols + pos;
+            do {
+                cell = d->grid[gridArrayOffset];
+                logicalGridIndex = d->findCellIndex(cell);
+                gridArrayOffset++;
+            } while (logicalGridIndex < logicalGridIndexBeforePosition
+                     && gridArrayOffset < d->nRows*d->nCols);
+
+            if (logicalGridIndex < logicalGridIndexBeforePosition
+                && gridArrayOffset == d->nRows*d->nCols)
+                cell = d->fragment_end;
+        }
+
         if (pos > 0 && pos < d->nCols && cell == d->grid[i*d->nCols + pos - 1]) {
             // cell spans the insertion place, extend it
-            fmt.setTableCellColumnSpan(fmt.tableCellColumnSpan() + num);
-            p->setCharFormat(it.position(), 1, fmt);
+            if (!extendedSpans.contains(cell)) {
+                QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
+                QTextCharFormat fmt = c->charFormat(it->format);
+                fmt.setTableCellColumnSpan(fmt.tableCellColumnSpan() + num);
+                p->setCharFormat(it.position(), 1, fmt);
+                d->dirty = true;
+                extendedSpans << cell;
+            }
         } else {
+            /* If the next cell is spanned from the row above, we need to find the right position
+            to insert to */
+            if (i > 0 && pos < d->nCols && cell == d->grid[(i-1) * d->nCols + pos]) {
+                int gridIndex = i*d->nCols + pos;
+                const int gridEnd = d->nRows * d->nCols - 1;
+                while (gridIndex < gridEnd && cell == d->grid[gridIndex]) {
+                    ++gridIndex;
+                }
+                if (gridIndex == gridEnd)
+                    cell = d->fragment_end;
+                else
+                    cell = d->grid[gridIndex];
+            }
+            QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
+            QTextCharFormat fmt = c->charFormat(it->format);
             fmt.setTableCellRowSpan(1);
             fmt.setTableCellColumnSpan(1);
             Q_ASSERT(fmt.objectIndex() == objectIndex());
@@ -915,12 +955,13 @@ void QTextTable::removeColumns(int pos, int num)
     for (int r = 0; r < d->nRows; ++r) {
         for (int c = pos; c < pos + num; ++c) {
             int cell = d->grid[r*d->nCols + c];
-            if (touchedCells.contains(cell))
-                continue;
-            touchedCells << cell;
             QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
             QTextCharFormat fmt = collection->charFormat(it->format);
             int span = fmt.tableCellColumnSpan();
+            if (touchedCells.contains(cell) && span <= 1)
+                continue;
+            touchedCells << cell;
+
             if (span > 1) {
                 fmt.setTableCellColumnSpan(span - 1);
                 p->setCharFormat(it.position(), 1, fmt);
