@@ -1,8 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -35,6 +34,7 @@
 **
 **
 **
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -694,6 +694,8 @@ void QLineControl::internalSetText(const QString &txt, int pos, bool edited)
 #ifndef QT_NO_ACCESSIBILITY
     if (changed)
         QAccessible::updateAccessibility(parent(), 0, QAccessible::TextUpdated);
+#else
+    Q_UNUSED(changed);
 #endif
 }
 
@@ -1200,6 +1202,13 @@ void QLineControl::internalUndo(int until)
         return;
     cancelPasswordEchoTimer();
     internalDeselect();
+
+    // Undo works only for clearing the line when in any of password the modes
+    if (m_echoMode != QLineEdit::Normal) {
+        clear();
+        return;
+    }
+
     while (m_undoState && m_undoState > until) {
         Command& cmd = m_history[--m_undoState];
         switch (cmd.type) {
@@ -1889,6 +1898,21 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
         event->accept();
 }
 
+bool QLineControl::isUndoAvailable() const
+{
+    // For security reasons undo is not available in any password mode (NoEcho included)
+    // with the exception that the user can clear the password with undo.
+    return !m_readOnly && m_undoState
+            && (m_echoMode == QLineEdit::Normal || m_history[m_undoState - 1].type == QLineControl::Insert);
+}
+
+bool QLineControl::isRedoAvailable() const
+{
+    // Same as with undo. Disabled for password modes.
+    return !m_readOnly
+            && m_echoMode == QLineEdit::Normal
+            && m_undoState < m_history.size();
+}
 
 QT_END_NAMESPACE
 
