@@ -1,8 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -35,6 +34,7 @@
 **
 **
 **
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -1544,6 +1544,28 @@ QAccessible::Role QAccessibleTabBar::role(int child) const
     return PageTab;
 }
 
+/*! \reimp */
+int QAccessibleTabBar::navigate(RelationFlag relation, int entry, QAccessibleInterface** target) const
+{
+    //QAccessibleWidgetEx::navigate might think that this is not a complex widget
+    //if close buttons are enabled, so this functions handles those cases in which
+    //the value to return is different if it's a complex widget and if it's not.
+    if (!target)
+        return -1;
+
+    *target = 0;
+
+    switch (relation) {
+    case Child:
+        if ((entry >= 0) && (entry <= childCount()))
+            return entry;
+        return -1;
+    default:
+        return QAccessibleWidgetEx::navigate(relation, entry, target);
+    }
+}
+
+
 /*! \reimp */
 QAccessible::State QAccessibleTabBar::state(int child) const
 {
@@ -1574,18 +1596,59 @@ QAccessible::State QAccessibleTabBar::state(int child) const
     else
         st |= Selectable;
 
-    if (!tb->currentIndex() == child - 1)
+    if (tb->currentIndex() == child - 1)
         st |= Selected;
 
     return st;
 }
 
+/*! \reimp */
+QString QAccessibleTabBar::actionText(int action, Text t, int child) const
+{
+    if (!child)
+        return QString();
+
+    switch (t) {
+    case QAccessible::Name:
+        if ((action == 1) && (child <= tabBar()->count())) {
+            return tabBar()->tabsClosable() ? QTabBar::tr("Close") : QString();
+        } else if (action == 0) {
+            if (child <= tabBar()->count())
+                return QTabBar::tr("Activate");
+            else //it's an scroll button
+                return QTabBar::tr("Press");
+        }
+        break;
+    case QAccessible::Description:
+        if ((action == 1) && (child <= tabBar()->count())) {
+            return tabBar()->tabsClosable() ? QTabBar::tr("Close the tab") : QString();
+        } else if (action == 0) {
+            if (child <= tabBar()->count())
+                return QTabBar::tr("Activate the tab");
+        }
+        break;
+    }
+    return QString();
+}
+
+/*! \reimp */
+int QAccessibleTabBar::userActionCount(int child) const
+{
+    if (!child || (child > tabBar()->count()))
+        return 0;
+    return tabBar()->tabsClosable() ? 1 : 0;
+}
+
 /*! \reimp */
 bool QAccessibleTabBar::doAction(int action, int child, const QVariantList &)
 {
     if (!child)
         return false;
 
+    if ((action == 1) && (child <= tabBar()->count()) && tabBar()->tabsClosable()) {
+        emit tabBar()->tabCloseRequested(child - 1);
+    }
+
     if (action != QAccessible::DefaultAction && action != QAccessible::Press)
         return false;
 
