@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -106,7 +106,7 @@ if (QLatin1String(signal) == SIGNAL(propertyChanged(QString,QDBusVariant))) {
 
 
         QObject::connect(helper,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)),
-                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)));
+                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)), Qt::UniqueConnection);
     }
 }
 
@@ -216,7 +216,6 @@ void QConnmanManagerInterface::registerCounter(const QString &path, quint32 inte
 {   QDBusReply<QList<QDBusObjectPath> > reply =  this->call(QLatin1String("RegisterCounter"),
                                                             QVariant::fromValue(path),
                                                             QVariant::fromValue(interval));
-    bool ok = true;
     if(reply.error().type() == QDBusError::InvalidArgs) {
         qWarning() << reply.error().message();
     }
@@ -225,7 +224,6 @@ void QConnmanManagerInterface::registerCounter(const QString &path, quint32 inte
 void QConnmanManagerInterface::unregisterCounter(const QString &path)
 {   QDBusReply<QList<QDBusObjectPath> > reply =  this->call(QLatin1String("UnregisterCounter"),
                                                             QVariant::fromValue(path));
-    bool ok = true;
     if(reply.error().type() == QDBusError::InvalidArgs) {
         qWarning() << reply.error().message();
     }
@@ -319,126 +317,8 @@ QString QConnmanManagerInterface::getPathForTechnology(const QString &name)
     return "";
 }
 
-QConnmanNetworkInterface::QConnmanNetworkInterface(const QString &dbusPathName, QObject *parent)
-    : QDBusAbstractInterface(QLatin1String(CONNMAN_SERVICE),
-                             dbusPathName,
-                             CONNMAN_NETWORK_INTERFACE,
-                             QDBusConnection::systemBus(), parent)
-{
-}
-
-QConnmanNetworkInterface::~QConnmanNetworkInterface()
-{
-}
-
-void QConnmanNetworkInterface::connectNotify(const char *signal)
-{
-    if (QLatin1String(signal) == SIGNAL(propertyChanged(QString,QDBusVariant))) {
-        if(!connection().connect(QLatin1String(CONNMAN_SERVICE),
-                               this->path(),
-                               QLatin1String(CONNMAN_NETWORK_INTERFACE),
-                               QLatin1String("PropertyChanged"),
-                               this,SIGNAL(propertyChanged(QString,QDBusVariant))) ) {
-            qWarning() << "network properties not connected";
-        }
-    }
-    if (QLatin1String(signal) == SIGNAL(propertyChangedContext(QString,QString,QDBusVariant))) {
-        QConnmanDBusHelper *helper;
-        helper = new QConnmanDBusHelper(this);
-
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
-                               this->path(),
-                               QLatin1String(CONNMAN_NETWORK_INTERFACE),
-                               QLatin1String("PropertyChanged"),
-                               helper,SLOT(propertyChanged(QString,QDBusVariant)));
-
-        QObject::connect(helper,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)),
-                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)));
-    }
-}
-
-void QConnmanNetworkInterface::disconnectNotify(const char *signal)
-{
-    if (QLatin1String(signal) == SIGNAL(propertyChanged(QString,QDBusVariant))) {
-
-    }
-}
-
-QVariantMap QConnmanNetworkInterface::getProperties()
-{
-    QDBusReply<QVariantMap > reply = this->call(QLatin1String("GetProperties"));
-    return reply.value();
-}
-
-QVariant QConnmanNetworkInterface::getProperty(const QString &property)
-{
-    QVariant var;
-    QVariantMap map = getProperties();
-    if (map.contains(property)) {
-        var = map.value(property);
-    }
-    return var;
-}
-
-//properties
-
-QString QConnmanNetworkInterface::getAddress()
-{
-    QVariant var = getProperty("Address");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanNetworkInterface::getName()
-{
-    QVariant var = getProperty("Name");
-    return qdbus_cast<QString>(var);
-}
-
-bool QConnmanNetworkInterface::isConnected()
-{
-    QVariant var = getProperty("Connected");
-    return qdbus_cast<bool>(var);
-}
-
-quint8 QConnmanNetworkInterface::getSignalStrength()
-{
-    QVariant var = getProperty("Strength");
-    return qdbus_cast<quint8>(var);
-}
-
-QString QConnmanNetworkInterface::getDevice()
-{
-    QVariant var = getProperty("Device");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanNetworkInterface::getWifiSsid()
-{
-    QVariant var = getProperty("WiFi.SSID");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanNetworkInterface::getWifiMode()
-{
-    QVariant var = getProperty("WiFi.Mode");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanNetworkInterface::getWifiSecurity()
-{
-    QVariant var = getProperty("WiFi.Security");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanNetworkInterface::getWifiPassphrase()
-{
-    QVariant var = getProperty("WiFi.Passphrase");
-    return qdbus_cast<QString>(var);
-}
-
 
 //////////////////////////
-
 QConnmanProfileInterface::QConnmanProfileInterface(const QString &dbusPathName,QObject *parent)
     : QDBusAbstractInterface(QLatin1String(CONNMAN_SERVICE),
                              dbusPathName,
@@ -505,6 +385,7 @@ QStringList QConnmanProfileInterface::getServices()
     return qdbus_cast<QStringList>(var);
 }
 
+
 ///////////////////////////
 QConnmanServiceInterface::QConnmanServiceInterface(const QString &dbusPathName,QObject *parent)
     : QDBusAbstractInterface(QLatin1String(CONNMAN_SERVICE),
@@ -538,7 +419,7 @@ void QConnmanServiceInterface::connectNotify(const char *signal)
                                helper,SLOT(propertyChanged(QString,QDBusVariant)));
 
         QObject::connect(helper,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)),
-                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)));
+                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)), Qt::UniqueConnection);
     }
 }
 
@@ -569,10 +450,9 @@ QVariant QConnmanServiceInterface::getProperty(const QString &property)
     return var;
 }
 
-// clearProperty
 void QConnmanServiceInterface::connect()
 {
-    QDBusReply<QVariantMap> reply = this->call(QLatin1String("Connect"));
+    this->asyncCall(QLatin1String("Connect"));
 }
 
 void QConnmanServiceInterface::disconnect()
@@ -866,7 +746,7 @@ void QConnmanTechnologyInterface::connectNotify(const char *signal)
                                helper,SLOT(propertyChanged(QString,QDBusVariant)));
 
         QObject::connect(helper,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)),
-                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)));
+                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)), Qt::UniqueConnection);
     }
 }
 
@@ -913,13 +793,6 @@ QString QConnmanTechnologyInterface::getType()
 }
 
 
-QStringList QConnmanTechnologyInterface::getDevices()
-{
-    QVariant var = getProperty("Devices");
-    return qdbus_cast<QStringList>(var);
-}
-
-
 //////////////////////////////////
 QConnmanAgentInterface::QConnmanAgentInterface(const QString &dbusPathName, QObject *parent)
     : QDBusAbstractInterface(QLatin1String(CONNMAN_SERVICE),
@@ -998,150 +871,6 @@ quint64 QConnmanCounterInterface::getTimeOnline()
 }
 
 /////////////////////////////////////////
-QConnmanDeviceInterface::QConnmanDeviceInterface(const QString &dbusPathName,QObject *parent)
-    : QDBusAbstractInterface(QLatin1String(CONNMAN_SERVICE),
-                             dbusPathName,
-                             CONNMAN_DEVICE_INTERFACE,
-                             QDBusConnection::systemBus(), parent)
-{
-}
-
-QConnmanDeviceInterface::~QConnmanDeviceInterface()
-{
-}
-
-void QConnmanDeviceInterface::connectNotify(const char *signal)
-{
-    if (QLatin1String(signal) == SIGNAL(propertyChanged(QString,QDBusVariant))) {
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
-                               this->path(),
-                               QLatin1String(CONNMAN_DEVICE_INTERFACE),
-                               QLatin1String("PropertyChanged"),
-                               this,SIGNAL(propertyChanged(QString,QDBusVariant)));
-
-    }
-    if (QLatin1String(signal) == SIGNAL(propertyChangedContext(QString,QString,QDBusVariant))) {
-        QConnmanDBusHelper *helper;
-        helper = new QConnmanDBusHelper(this);
-
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
-                               this->path(),
-                               QLatin1String(CONNMAN_DEVICE_INTERFACE),
-                               QLatin1String("PropertyChanged"),
-                               helper,SLOT(propertyChanged(QString,QDBusVariant)));
-
-        QObject::connect(helper,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)),
-                this,SIGNAL(propertyChangedContext(const QString &,const QString &,const QDBusVariant &)));
-    }
-}
-
-void QConnmanDeviceInterface::disconnectNotify(const char *signal)
-{
-    if (QLatin1String(signal) == SIGNAL(propertyChanged(QString,QVariant))) {
-
-    }
-}
-
-QVariantMap QConnmanDeviceInterface::getProperties()
-{
-    QDBusReply<QVariantMap> reply =  this->call(QLatin1String("GetProperties"));
-    return reply.value();
-}
-
-bool QConnmanDeviceInterface::setProperty(const QString &name, const QDBusVariant &value)
-{
-    QDBusMessage reply = this->call(QLatin1String("SetProperty"),name, qVariantFromValue(value));
-    return true;
-}
-
-void QConnmanDeviceInterface::scan()
-{
-    QDBusReply<QVariantMap> reply = this->call(QLatin1String("ProposeScan"));
-    if(!reply.isValid()) {
-        qDebug() << reply.error().message();
-    }
-}
-
-QVariant QConnmanDeviceInterface::getProperty(const QString &property)
-{
-    QVariant var;
-    QVariantMap map = getProperties();
-    if (map.contains(property)) {
-        var = map.value(property);
-    }
-    return var;
-}
-
-//properties
-QString QConnmanDeviceInterface::getAddress()
-{
-    QVariant var = getProperty("Address");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanDeviceInterface::getName()
-{
-    QVariant var = getProperty("Name");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanDeviceInterface::getType()
-{
-    QVariant var = getProperty("Type");
-    return qdbus_cast<QString>(var);
-}
-
-QString QConnmanDeviceInterface::getInterface()
-{
-    QVariant var = getProperty("Interface");
-    return qdbus_cast<QString>(var);
-}
-
-bool QConnmanDeviceInterface::isPowered()
-{
-    QVariant var = getProperty("Powered");
-    return qdbus_cast<bool>(var);
-}
-
-quint16 QConnmanDeviceInterface::getScanInterval()
-{
-    QVariant var = getProperty("ScanInterval");
-    return qdbus_cast<quint16>(var);
-}
-
-bool QConnmanDeviceInterface::setScanInterval(const QString & interval)
-{
-//    QList<QVariant> args;
-//    args << qVariantFromValue(name)
-//    << value.variant();
-
-//    QDBusMessage reply = this->callWithArgumentList(QDBus::AutoDetect,QLatin1String("SetProperty"),args);
-
-    return setProperty("ScanInterval", QDBusVariant(interval));
-}
-
-bool QConnmanDeviceInterface::isScanning()
-{
-    QVariant var = getProperty("Scanning");
-     return qdbus_cast<bool>(var);
-}
-
-QStringList QConnmanDeviceInterface::getNetworks()
-{
-    QVariant var = getProperty("Networks");
-    return qdbus_cast<QStringList>(var);
-}
-
-bool QConnmanDeviceInterface::setEnabled(bool powered)
-{
-    QList<QVariant> args;
-    args << qVariantFromValue(QString("Powered"))
-    << qVariantFromValue(QDBusVariant(powered));
-
-    QDBusMessage reply = this->callWithArgumentList(QDBus::AutoDetect,QLatin1String("SetProperty"),args);
-    return true;
-}
-
 QConnmanDBusHelper::QConnmanDBusHelper(QObject * parent)
         : QObject(parent)
 {
