@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -125,9 +125,6 @@ static QString get_network_interface();
 
 void QNetworkSessionPrivateImpl::iapStateChanged(const QString& iapid, uint icd_connection_state)
 {
-    if ((publicConfig.type() == QNetworkConfiguration::UserChoice) && opened) {
-        updateIdentifier(iapid);
-    }
 
     if (((publicConfig.type() == QNetworkConfiguration::UserChoice) &&
          (activeConfig.identifier() == iapid)) ||
@@ -149,6 +146,9 @@ void QNetworkSessionPrivateImpl::iapStateChanged(const QString& iapid, uint icd_
             break;
         }
     }
+    if (publicConfig.type() == QNetworkConfiguration::UserChoice) {
+        updateIdentifier(iapid);
+    }
 }
 
 void QNetworkSessionPrivateImpl::cleanupSession(void)
@@ -183,6 +183,12 @@ void QNetworkSessionPrivateImpl::updateState(QNetworkSession::State newState)
                 icdConfig->mutex.lock();
                 icdConfig->state = QNetworkConfiguration::Defined;
                 icdConfig->mutex.unlock();
+
+                // Reset the state of the default configuration to Discovered
+                icdConfig = toIcdConfig(privateConfiguration(publicConfig));
+                icdConfig->mutex.lock();
+                icdConfig->state = QNetworkConfiguration::Discovered;
+                icdConfig->mutex.unlock();
             } else {
                 if (!activeConfig.isValid()) {
                     // Active configuration (IAP) was removed from system
@@ -621,21 +627,21 @@ static QString get_network_interface()
     if (ret == 0) {
 	/* No results */
 #ifdef BEARER_MANAGEMENT_DEBUG
-	qDebug() << "Cannot get addrinfo from icd, are you connected or is icd running?";
+        qDebug() << "Cannot get addrinfo from icd, are you connected or is icd running?";
 #endif
-	return iface;
+        return iface;
     }
 
     if (addr_results.first().ip_info.isEmpty())
-	return QString();
+        return QString();
 
-    const char *address = addr_results.first().ip_info.first().address.toAscii().constData();
+    QByteArray data = addr_results.first().ip_info.first().address.toAscii();
     struct in_addr addr;
-    if (inet_aton(address, &addr) == 0) {
+    if (inet_aton(data.constData(), &addr) == 0) {
 #ifdef BEARER_MANAGEMENT_DEBUG
-	qDebug() << "address" << address << "invalid";
+        qDebug() << "address" << data.constData() << "invalid";
 #endif
-	return iface;
+        return iface;
     }
 
     struct ifaddrs *ifaddr, *ifa;
@@ -679,7 +685,7 @@ void QNetworkSessionPrivateImpl::open()
     if (serviceConfig.isValid()) {
         lastError = QNetworkSession::OperationNotSupportedError;
         emit QNetworkSessionPrivate::error(lastError);
-    } else if (!isOpen) {
+    } else if (!opened) {
 	if (publicConfig.type() == QNetworkConfiguration::UserChoice) {
 	    /* Caller is trying to connect to default IAP.
 	     * At this time we will not know the IAP details so we just
@@ -794,7 +800,7 @@ void QNetworkSessionPrivateImpl::stateChange(const QDBusMessage& rep)
             qDebug() << "connect to"<< publicConfig.identifier() << "failed, result is empty";
 #endif
             updateState(QNetworkSession::Disconnected);
-            emit QNetworkSessionPrivate::error(QNetworkSession::InvalidConfigurationError);
+            emit QNetworkSessionPrivate::error(QNetworkSession::SessionAbortedError);
             if (publicConfig.type() == QNetworkConfiguration::UserChoice)
                     copyConfig(publicConfig, activeConfig);
             return;
@@ -808,7 +814,7 @@ void QNetworkSessionPrivateImpl::stateChange(const QDBusMessage& rep)
         if ((publicConfig.type() != QNetworkConfiguration::UserChoice) &&
             (connected_iap != config.identifier())) {
             updateState(QNetworkSession::Disconnected);
-            emit QNetworkSessionPrivate::error(QNetworkSession::InvalidConfigurationError);
+            emit QNetworkSessionPrivate::error(QNetworkSession::UnknownSessionError);
             return;
         }
 
@@ -1026,6 +1032,9 @@ QString QNetworkSessionPrivateImpl::errorString() const
     case QNetworkSession::SessionAbortedError:
         errorStr = QNetworkSessionPrivateImpl::tr("Session aborted by user or system");
         break;
+    case QNetworkSession::InvalidConfigurationError:
+        errorStr = QNetworkSessionPrivateImpl::tr("The specified configuration cannot be used.");
+        break;
     default:
     case QNetworkSession::UnknownSessionError:
         errorStr = QNetworkSessionPrivateImpl::tr("Unidentified Error");
