@@ -1,44 +1,45 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+#include "qclucenefieldnames_p.h"
 #include "qhelpenginecore.h"
 #include "qhelp_global.h"
 #include "fulltextsearch/qhits_p.h"
@@ -63,9 +64,8 @@
 
 QT_BEGIN_NAMESPACE
 
-namespace qt {
-    namespace fulltextsearch {
-        namespace clucene {
+namespace fulltextsearch {
+namespace clucene {
 
 // taken from qtexthtmlparser
 static const struct QTextHtmlEntity
@@ -407,17 +407,17 @@ public:
             QString parsedTitle = QHelpGlobal::documentTitle(data);
 
             if(!parsedData.isEmpty()) {
-                document->add(new QCLuceneField(QLatin1String("content"),
+                document->add(new QCLuceneField(ContentField,
                     parsedData,QCLuceneField::INDEX_TOKENIZED));
-                document->add(new QCLuceneField(QLatin1String("path"), fileName,
+                document->add(new QCLuceneField(PathField, fileName,
                     QCLuceneField::STORE_YES | QCLuceneField::INDEX_UNTOKENIZED));
-                document->add(new QCLuceneField(QLatin1String("title"), parsedTitle,
+                document->add(new QCLuceneField(TitleField, parsedTitle,
                     QCLuceneField::STORE_YES | QCLuceneField::INDEX_UNTOKENIZED));
-                document->add(new QCLuceneField(QLatin1String("titleTokenized"), parsedTitle,
+                document->add(new QCLuceneField(TitleTokenizedField, parsedTitle,
                     QCLuceneField::STORE_YES | QCLuceneField::INDEX_TOKENIZED));
-                document->add(new QCLuceneField(QLatin1String("namespace"), namespaceName,
+                document->add(new QCLuceneField(NamespaceField, namespaceName,
                     QCLuceneField::STORE_YES | QCLuceneField::INDEX_UNTOKENIZED));
-                document->add(new QCLuceneField(QLatin1String("attribute"), attributes,
+                document->add(new QCLuceneField(AttributeField, attributes,
                     QCLuceneField::STORE_YES | QCLuceneField::INDEX_TOKENIZED));
                 return true;
             }
@@ -578,6 +578,7 @@ void QHelpSearchIndexWriter::cancelIndexing()
 void QHelpSearchIndexWriter::updateIndex(const QString &collectionFile,
     const QString &indexFilesFolder, bool reindex)
 {
+    wait();
     mutex.lock();
     this->m_cancel = false;
     this->m_reindex = reindex;
@@ -585,7 +586,7 @@ void QHelpSearchIndexWriter::updateIndex(const QString &collectionFile,
     this->m_indexFilesFolder = indexFilesFolder;
     mutex.unlock();
 
-    start(QThread::NormalPriority);
+    start(QThread::LowestPriority);
 }
 
 void QHelpSearchIndexWriter::optimizeIndex()
@@ -612,67 +613,69 @@ void QHelpSearchIndexWriter::optimizeIndex()
 
 void QHelpSearchIndexWriter::run()
 {
-    QMutexLocker mutexLocker(&mutex);
-
-    if (m_cancel)
-        return;
-
-    const bool reindex = this->m_reindex;
-    const QString collectionFile(this->m_collectionFile);
-
-    mutexLocker.unlock();
-
-    QHelpEngineCore engine(collectionFile, 0);
-    if (!engine.setupData())
-        return;
-
-    const QLatin1String key("CluceneIndexedNamespaces");
-    if (reindex)
-        engine.setCustomValue(key, QLatin1String(""));
-
-    QMap<QString, QDateTime> indexMap;
-    const QLatin1String oldKey("CluceneSearchNamespaces");
-    if (!engine.customValue(oldKey, QString()).isNull()) {
-        // old style qhc file < 4.4.2, need to convert...
-        const QStringList indexedNamespaces = engine.customValue(oldKey).
-            toString().split(QLatin1String("|"), QString::SkipEmptyParts);
-        foreach (const QString &nameSpace, indexedNamespaces)
-            indexMap.insert(nameSpace, QDateTime());
-        engine.removeCustomValue(oldKey);
-    } else {
-        QDataStream dataStream(engine.customValue(key).toByteArray());
-        dataStream >> indexMap;
-    }
-
-    QString indexPath = m_indexFilesFolder;
-
-    QFileInfo fInfo(indexPath);
-    if (fInfo.exists() && !fInfo.isWritable()) {
-        qWarning("Full Text Search, could not create index (missing permissions for '%s').", qPrintable(indexPath));
-        return;
-    }
-
-    emit indexingStarted();
-
-    QCLuceneIndexWriter *writer = 0;
-    QCLuceneStandardAnalyzer analyzer;
-    const QStringList registeredDocs = engine.registeredDocumentations();
-
-    QLocalSocket localSocket;
-    localSocket.connectToServer(QString(QLatin1String("QtAssistant%1"))
-        .arg(QLatin1String(QT_VERSION_STR)));
-
-    QLocalServer localServer;
-    bool otherInstancesRunning = true;
-    if (!localSocket.waitForConnected()) {
-        otherInstancesRunning = false;
-        localServer.listen(QString(QLatin1String("QtAssistant%1"))
-            .arg(QLatin1String(QT_VERSION_STR)));
-    }
-
 #if !defined(QT_NO_EXCEPTIONS)
     try {
 #endif
+        QMutexLocker mutexLocker(&mutex);
+
+        if (m_cancel)
+            return;
+
+        const bool reindex = this->m_reindex;
+        const QString collectionFile(this->m_collectionFile);
+
+        mutexLocker.unlock();
+
+        QHelpEngineCore engine(collectionFile, 0);
+        if (!engine.setupData())
+            return;
+
+        const QLatin1String key("CluceneIndexedNamespaces");
+        if (reindex)
+            engine.setCustomValue(key, QLatin1String(""));
+
+        QMap<QString, QDateTime> indexMap;
+        const QLatin1String oldKey("CluceneSearchNamespaces");
+        if (!engine.customValue(oldKey, QString()).isNull()) {
+            // old style qhc file < 4.4.2, need to convert...
+            const QStringList indexedNamespaces
+                = engine.customValue(oldKey).toString()
+                  .split(QLatin1String("|"), QString::SkipEmptyParts);
+            foreach (const QString &nameSpace, indexedNamespaces)
+                indexMap.insert(nameSpace, QDateTime());
+            engine.removeCustomValue(oldKey);
+        } else {
+            QDataStream dataStream(engine.customValue(key).toByteArray());
+            dataStream >> indexMap;
+        }
+
+        QString indexPath = m_indexFilesFolder;
+
+        QFileInfo fInfo(indexPath);
+        if (fInfo.exists() && !fInfo.isWritable()) {
+            qWarning("Full Text Search, could not create index (missing permissions for '%s').",
+                     qPrintable(indexPath));
+            return;
+        }
+
+        emit indexingStarted();
+
+        QCLuceneIndexWriter *writer = 0;
+        QCLuceneStandardAnalyzer analyzer;
+        const QStringList registeredDocs = engine.registeredDocumentations();
+
+        QLocalSocket localSocket;
+        localSocket.connectToServer(QString(QLatin1String("QtAssistant%1"))
+                                    .arg(QLatin1String(QT_VERSION_STR)));
+
+        QLocalServer localServer;
+        bool otherInstancesRunning = true;
+        if (!localSocket.waitForConnected()) {
+            otherInstancesRunning = false;
+            localServer.listen(QString(QLatin1String("QtAssistant%1"))
+                               .arg(QLatin1String(QT_VERSION_STR)));
+        }
+
         // check if it's locked, and if the other instance is running
         if (!otherInstancesRunning && QCLuceneIndexReader::isLocked(indexPath))
             QCLuceneIndexReader::unlock(indexPath);
@@ -704,7 +707,8 @@ void QHelpSearchIndexWriter::run()
                     removeDocuments(indexPath, namespaceName);
                 } else {
                     QString path = engine.documentationFileName(namespaceName);
-                    if (indexMap.value(namespaceName) < QFileInfo(path).lastModified()) {
+                    if (indexMap.value(namespaceName)
+                        < QFileInfo(path).lastModified()) {
                         // make sure we remove some outdated indexed stuff
                         indexMap.remove(namespaceName);
                         removeDocuments(indexPath, namespaceName);
@@ -712,9 +716,7 @@ void QHelpSearchIndexWriter::run()
 
                     if (indexMap.contains(namespaceName)) {
                         // make sure we really have content indexed for namespace
-                        // NOTE: Extra variable just for GCC 3.3.5
-                        QLatin1String key("namespace");
-                        QCLuceneTermQuery query(QCLuceneTerm(key, namespaceName));
+                        QCLuceneTermQuery query(QCLuceneTerm(NamespaceField, namespaceName));
                         QCLuceneIndexSearcher indexSearcher(indexPath);
                         QCLuceneHits hits = indexSearcher.search(query);
                         if (hits.length() <= 0)
@@ -727,94 +729,85 @@ void QHelpSearchIndexWriter::run()
             indexMap.clear();
             writer = new QCLuceneIndexWriter(indexPath, analyzer, true);
         }
-#if !defined(QT_NO_EXCEPTIONS)
-    } catch (...) {
-        qWarning("Full Text Search, could not create index writer in '%s'.",
-            qPrintable(indexPath));
-        return;
-    }
-#endif
 
-#if !defined(QT_NO_EXCEPTIONS)
-    try {
-#endif
         writer->setMergeFactor(100);
         writer->setMinMergeDocs(1000);
         writer->setMaxFieldLength(QCLuceneIndexWriter::DEFAULT_MAX_FIELD_LENGTH);
-#if !defined(QT_NO_EXCEPTIONS)
-    } catch (...) {
-        qWarning("Full Text Search, could not set writer properties.");
-        return;
-    }
-#endif
 
-    QStringList namespaces;
-    foreach(const QString &namespaceName, registeredDocs) {
-        mutexLocker.relock();
-        if (m_cancel) {
-            closeIndexWriter(writer);
-            emit indexingFinished();
-            return;
-        }
-        mutexLocker.unlock();
-
-        namespaces.append(namespaceName);
-        if (indexMap.contains(namespaceName))
-            continue;
-
-        const QList<QStringList> attributeSets =
-            engine.filterAttributeSets(namespaceName);
-
-        if (attributeSets.isEmpty()) {
-            const QList<QUrl> docFiles = indexableFiles(&engine, namespaceName,
-                QStringList());
-            if (!addDocuments(docFiles, engine, QStringList(), namespaceName,
-                writer, analyzer))
-                break;
-        } else {
-            bool bail = false;
-            foreach (const QStringList &attributes, attributeSets) {
-                const QList<QUrl> docFiles = indexableFiles(&engine,
-                    namespaceName, attributes);
-                if (!addDocuments(docFiles, engine, attributes, namespaceName,
-                    writer, analyzer)) {
-                    bail = true;
-                    break;
-                }
+        QStringList namespaces;
+        foreach(const QString &namespaceName, registeredDocs) {
+            mutexLocker.relock();
+            if (m_cancel) {
+                closeIndexWriter(writer);
+                emit indexingFinished();
+                return;
             }
-            if (bail)
-                break;
+            mutexLocker.unlock();
+
+            namespaces.append(namespaceName);
+            if (indexMap.contains(namespaceName))
+                continue;
+
+            const QList<QStringList> attributeSets =
+                    engine.filterAttributeSets(namespaceName);
+
+            if (attributeSets.isEmpty()) {
+                const QList<QUrl> docFiles = indexableFiles(&engine, namespaceName,
+                                                            QStringList());
+                if (!addDocuments(docFiles, engine, QStringList(), namespaceName,
+                                  writer, analyzer))
+                    break;
+            } else {
+                bool bail = false;
+                foreach (const QStringList &attributes, attributeSets) {
+                    const QList<QUrl> docFiles = indexableFiles(&engine,
+                                                                namespaceName, attributes);
+                    if (!addDocuments(docFiles, engine, attributes, namespaceName,
+                                      writer, analyzer)) {
+                        bail = true;
+                        break;
+                    }
+                }
+                if (bail)
+                    break;
+            }
+
+            mutexLocker.relock();
+            if (!m_cancel) {
+                QString path(engine.documentationFileName(namespaceName));
+                indexMap.insert(namespaceName, QFileInfo(path).lastModified());
+                writeIndexMap(engine, indexMap);
+            }
+            mutexLocker.unlock();
         }
 
+        closeIndexWriter(writer);
+
         mutexLocker.relock();
         if (!m_cancel) {
-            QString path(engine.documentationFileName(namespaceName));
-            indexMap.insert(namespaceName, QFileInfo(path).lastModified());
-            writeIndexMap(engine, indexMap);
-        }
-        mutexLocker.unlock();
-    }
-
-    closeIndexWriter(writer);
-
-    mutexLocker.relock();
-    if (!m_cancel) {
-        mutexLocker.unlock();
-
-        QStringList indexedNamespaces = indexMap.keys();
-        foreach(const QString &namespaceName, indexedNamespaces) {
-            mutexLocker.relock();
-            if (m_cancel)
-                break;
             mutexLocker.unlock();
 
-            if (!namespaces.contains(namespaceName)) {
-                indexMap.remove(namespaceName);
-                writeIndexMap(engine, indexMap);
-                removeDocuments(indexPath, namespaceName);
+            QStringList indexedNamespaces = indexMap.keys();
+            foreach(const QString &namespaceName, indexedNamespaces) {
+                mutexLocker.relock();
+                if (m_cancel)
+                    break;
+                mutexLocker.unlock();
+
+                if (!namespaces.contains(namespaceName)) {
+                    indexMap.remove(namespaceName);
+                    writeIndexMap(engine, indexMap);
+                    removeDocuments(indexPath, namespaceName);
+                }
             }
         }
+
+#if !defined(QT_NO_EXCEPTIONS)
+    } catch (...) {
+        qWarning("%s: Failed because of CLucene exception.", Q_FUNC_INFO);
     }
+#endif
+
     emit indexingFinished();
 }
 
@@ -857,8 +850,7 @@ void QHelpSearchIndexWriter::removeDocuments(const QString &indexPath,
         return;
 
     QCLuceneIndexReader reader = QCLuceneIndexReader::open(indexPath);
-    reader.deleteDocuments(QCLuceneTerm(QLatin1String("namespace"),
-        namespaceName));
+    reader.deleteDocuments(QCLuceneTerm(NamespaceField, namespaceName));
 
     reader.close();
 }
@@ -900,8 +892,7 @@ void QHelpSearchIndexWriter::closeIndexWriter(QCLuceneIndexWriter *writer)
 #endif
 }
 
-        }   // namespace clucene
-    }   // namespace fulltextsearch
-}   // namespace qt
+}   // namespace clucene
+}   // namespace fulltextsearch
 
 QT_END_NAMESPACE
